import * as i0 from '@angular/core';
import { Injectable, EventEmitter, Component, Input, Output, ViewChild, HostListener, NgModule } from '@angular/core';
import { pascalCase } from 'pascal-case';
import { parse } from 'yaml';
import { Client } from 'typehub-javascript-sdk';
import { lastValueFrom, firstValueFrom } from 'rxjs';
import * as i1 from '@angular/common/http';
import * as i4 from '@ng-bootstrap/ng-bootstrap';
import { NgbModule } from '@ng-bootstrap/ng-bootstrap';
import * as i5 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i2 from '@angular/forms';
import { FormsModule } from '@angular/forms';

class ExportService {
    constructor() { }
    transform(spec) {
        const schema = {};
        if (spec.baseUrl !== undefined) {
            schema.baseUrl = spec.baseUrl;
        }
        if (spec.security !== undefined) {
            schema.security = spec.security;
        }
        if (spec.imports && Array.isArray(spec.imports) && spec.imports.length > 0) {
            const imports = {};
            spec.imports.forEach((include) => {
                if (include.alias && include.url) {
                    imports[include.alias] = include.url;
                }
            });
            schema['import'] = imports;
        }
        if (spec.operations && Array.isArray(spec.operations) && spec.operations.length > 0) {
            const operations = {};
            spec.operations.forEach((operation) => {
                operations[operation.name] = this.transformOperation(operation);
            });
            schema.operations = operations;
        }
        if (spec.types && Array.isArray(spec.types) && spec.types.length > 0) {
            const definitions = {};
            spec.types.forEach((type) => {
                if (type.type === 'map') {
                    definitions[type.name] = this.transformMap(type);
                }
                else {
                    definitions[type.name] = this.transformStruct(type);
                }
            });
            schema.definitions = definitions;
        }
        if (spec.root !== undefined && spec.types[spec.root]) {
            schema['root'] = spec.types[spec.root].name;
        }
        return schema;
    }
    transformOperation(operation) {
        const result = {};
        if (this.isset(operation.description)) {
            result.description = operation.description;
        }
        if (this.isset(operation.httpMethod)) {
            result.method = operation.httpMethod;
        }
        if (this.isset(operation.httpPath)) {
            result.path = operation.httpPath;
        }
        const args = {};
        if (this.isset(operation.arguments) && Array.isArray(operation.arguments)) {
            operation.arguments.forEach((argument) => {
                if (argument.in === 'path') {
                    const schema = this.resolveType(argument.type);
                    if (schema === null) {
                        return;
                    }
                    args[argument.name] = {
                        in: 'path',
                        schema: schema,
                    };
                }
            });
        }
        if (this.isset(operation.payload) && operation.payload) {
            const payload = {
                in: 'body',
            };
            if (operation.payloadShape === 'mime') {
                payload.contentType = operation.payload;
            }
            else {
                payload.schema = this.getSchemaForShape(operation.payload, operation.payloadShape);
            }
            args['payload'] = payload;
        }
        if (this.isset(operation.arguments) && Array.isArray(operation.arguments) && operation.arguments.length > 0) {
            operation.arguments.forEach((argument) => {
                if (argument.in === 'query' || argument.in === 'header') {
                    const schema = this.resolveType(argument.type);
                    if (schema === null) {
                        return;
                    }
                    args[argument.name] = {
                        in: argument.in,
                        schema: schema,
                    };
                }
            });
        }
        if (Object.entries(args).length > 0) {
            result.arguments = args;
        }
        if (this.isset(operation.throws) && Array.isArray(operation.throws) && operation.throws.length > 0) {
            const throws = [];
            operation.throws.forEach((throw_) => {
                const ret = {
                    code: throw_.code,
                };
                if (throw_.typeShape === 'mime') {
                    ret.contentType = throw_.type;
                }
                else {
                    ret.schema = this.getSchemaForShape(throw_.type, throw_.typeShape);
                }
                throws.push(ret);
            });
            result.throws = throws;
        }
        let httpCode = 200;
        if (this.isset(operation.httpCode)) {
            httpCode = operation.httpCode;
        }
        if (this.isset(operation.return) && httpCode !== 204) {
            const ret = {};
            ret.code = httpCode;
            if (operation.returnShape === 'mime') {
                ret.contentType = operation.return;
            }
            else {
                ret.schema = this.getSchemaForShape(operation.return, operation.returnShape);
            }
            result.return = ret;
        }
        return result;
    }
    getSchemaForShape(type, shape) {
        if (shape === 'map') {
            return {
                type: 'map',
                schema: {
                    type: 'reference',
                    target: type
                }
            };
        }
        else if (shape === 'array') {
            return {
                type: 'array',
                schema: {
                    type: 'reference',
                    target: type
                }
            };
        }
        else {
            return {
                type: 'reference',
                target: type
            };
        }
    }
    transformStruct(type) {
        const result = {};
        if (this.isset(type.description)) {
            result['description'] = type.description;
        }
        if (this.isset(type.deprecated)) {
            result['deprecated'] = type.deprecated;
        }
        result['type'] = 'struct';
        if (this.isset(type.parent)) {
            const parent = {};
            parent['type'] = 'reference';
            parent['target'] = type.parent;
            if (this.isset(type.template) && type.template && Object.keys(type.template).length > 0) {
                parent['template'] = type.template;
            }
            result['parent'] = parent;
        }
        if (this.isset(type.base)) {
            result['base'] = type.base;
        }
        if (type.properties && type.properties.length > 0) {
            const props = {};
            type.properties.forEach((property) => {
                props[property.name] = this.generateProperty(property);
            });
            result['properties'] = props;
        }
        if (this.isset(type.discriminator)) {
            result['discriminator'] = type.discriminator;
        }
        if (this.isset(type.mapping) && type.mapping && Object.keys(type.mapping).length > 0) {
            result['mapping'] = type.mapping;
        }
        return result;
    }
    transformMap(type) {
        const result = {};
        if (this.isset(type.description)) {
            result['description'] = type.description;
        }
        if (this.isset(type.deprecated)) {
            result['deprecated'] = type.deprecated;
        }
        result['type'] = 'map';
        if (this.isset(type.reference)) {
            result['schema'] = this.resolveType(type.reference);
        }
        return result;
    }
    generateProperty(property) {
        const result = {};
        if (this.isset(property.description)) {
            result['description'] = property.description;
        }
        if (this.isset(property.deprecated)) {
            result['deprecated'] = property.deprecated;
        }
        const reference = property.reference;
        const generic = property.generic;
        if (property.type === 'reference' || property.type === 'object') {
            result['type'] = 'reference';
            result['target'] = property.reference;
            if (this.isset(property.template) && property.template && Object.keys(property.template).length > 0) {
                result['template'] = property.template;
            }
        }
        else if (property.type === 'map' && reference) {
            result['type'] = 'map';
            result['schema'] = this.resolveType(reference, generic);
        }
        else if (property.type === 'array' && reference) {
            result['type'] = 'array';
            result['schema'] = this.resolveType(reference, generic);
        }
        else if (property.type === 'string') {
            result['type'] = 'string';
            if (this.isset(property.format)) {
                result['format'] = property.format;
            }
        }
        else if (property.type === 'integer') {
            result['type'] = 'integer';
        }
        else if (property.type === 'number') {
            result['type'] = 'number';
        }
        else if (property.type === 'boolean') {
            result['type'] = 'boolean';
        }
        else if (property.type === 'any') {
            result['type'] = 'any';
        }
        else if (property.type === 'generic') {
            result['type'] = 'generic';
            result['name'] = property.generic;
        }
        return result;
    }
    resolveType(reference, generic) {
        if (!reference) {
            return null;
        }
        if (['string', 'integer', 'number', 'boolean', 'any'].includes(reference)) {
            return {
                type: reference
            };
        }
        else if (reference === 'generic') {
            return {
                type: 'generic',
                name: generic
            };
        }
        else {
            return {
                type: 'reference',
                target: reference
            };
        }
    }
    isset(value) {
        return typeof value !== 'undefined' && value !== null;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ExportService, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
    static ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ExportService, providedIn: 'root' });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ExportService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [] });

class Internal {
    bcLayerService;
    constructor(bcLayerService) {
        this.bcLayerService = bcLayerService;
    }
    async transform(schema) {
        let spec = JSON.parse(schema);
        spec = this.bcLayerService.transform(spec);
        return spec;
    }
}

class TypeSchema {
    typeHubService;
    resolverService;
    anonymousObjects = {};
    scalarTypes = ['string', 'integer', 'number', 'boolean', 'any'];
    constructor(typeHubService, resolverService) {
        this.typeHubService = typeHubService;
        this.resolverService = resolverService;
    }
    async transform(schema) {
        return this.build(JSON.parse(schema));
    }
    async build(data) {
        this.anonymousObjects = {};
        const typeNames = [];
        const spec = {
            imports: [],
            operations: [],
            types: []
        };
        const rawImport = this.get(data, ['import', '$import']);
        if (typeof rawImport === 'object') {
            for (const [key, value] of Object.entries(rawImport)) {
                const include = await this.transformImport(key, value);
                if (include) {
                    spec.imports.push(include);
                }
            }
        }
        const definitions = this.get(data, ['definitions', '$defs']);
        if (typeof definitions === 'object') {
            for (const [key, value] of Object.entries(definitions)) {
                try {
                    spec.types.push(await this.transformType(key, value));
                    typeNames.push(key);
                }
                catch (error) {
                }
            }
        }
        if (this.isset(data['components']) && this.isset(data['components']['schemas']) && typeof data['components']['schemas'] === 'object') {
            for (const [key, value] of Object.entries(data['components']['schemas'])) {
                try {
                    spec.types.push(await this.transformType(key, value));
                    typeNames.push(key);
                }
                catch (error) {
                }
            }
        }
        for (const [key, value] of Object.entries(this.anonymousObjects)) {
            try {
                spec.types.push(await this.transformType(key, value));
                typeNames.push(key);
            }
            catch (error) {
            }
        }
        if (this.isset(data['properties']) && typeof data['properties'] === 'object') {
            // it looks like we have a root data we try to convert
            let key = 'Root';
            if (this.isset(data['title']) && typeof data['title'] === 'string') {
                key = pascalCase(data['title']);
            }
            spec.types.push(await this.transformType(key, {
                description: data['description'],
                type: 'object',
                properties: data['properties'],
            }));
            typeNames.push(key);
            spec.root = spec.types.length - 1;
        }
        const root = this.get(data, ['root', '$ref']);
        if (root === 'string') {
            const index = typeNames.indexOf(root);
            if (index !== -1) {
                spec.root = index;
            }
        }
        return spec;
    }
    async transformImport(alias, url) {
        if (typeof url !== 'string') {
            return;
        }
        let include = {
            alias: alias,
            url: url,
        };
        include.types = await this.resolverService.resolveIncludeTypes(include);
        return include;
    }
    async transformType(name, data) {
        let type;
        if (this.isset(data['$ref'])) {
            data['type'] = 'reference';
            data['target'] = data['$ref'];
        }
        else if (this.isset(data['additionalProperties'])) {
            data['type'] = 'map';
            data['schema'] = data['additionalProperties'];
        }
        else if (this.isset(data['items'])) {
            data['type'] = 'array';
            data['schema'] = data['items'];
        }
        if (this.isset(data['$extends'])) {
            data['extends'] = {
                type: 'reference',
                target: data['$extends'],
            };
        }
        if (this.isset(data['$template'])) {
            data['template'] = data['$template'];
        }
        const typeName = data['type'];
        if (typeName === 'reference') {
            type = {
                type: 'reference',
                name: name,
                description: data['description'] && typeof data['description'] === 'string' ? data['description'] : '',
                reference: data['target']
            };
            if (this.isset(data['template'])) {
                type.template = data['template'];
            }
        }
        else if (typeName === 'map' || typeName === 'array') {
            type = {
                type: typeName,
                name: name,
                description: data['description'] && typeof data['description'] === 'string' ? data['description'] : '',
                reference: this.parseRef(data['schema'])
            };
        }
        else {
            if (typeName && this.scalarTypes.includes(typeName)) {
                throw new Error('Can not create scalar object');
            }
            type = {
                type: 'struct',
                name: name,
                description: data['description'] && typeof data['description'] === 'string' ? data['description'] : '',
            };
            if (this.isset(data['extends']) && this.isset(data['extends'].target) && typeof data['extends'].target === 'string') {
                type.parent = data['extends'].target;
            }
            if (this.isset(data['properties']) && typeof data['properties'] === 'object') {
                type.properties = [];
                for (const [key, value] of Object.entries(data['properties'])) {
                    try {
                        type.properties.push(await this.transformProperty(key, value));
                    }
                    catch (error) {
                    }
                }
            }
        }
        return type;
    }
    async transformProperty(name, data) {
        if (this.isset(data['$ref'])) {
            data['type'] = 'reference';
            data['target'] = data['$ref'];
        }
        else if (this.isset(data['additionalProperties'])) {
            data['type'] = 'map';
            data['schema'] = data['additionalProperties'];
        }
        else if (this.isset(data['items'])) {
            data['type'] = 'array';
            data['schema'] = data['items'];
        }
        const typeName = data['type'];
        let type;
        let ref;
        let format;
        if (this.isset(data['properties'])) {
            // in this case we have a nested object which is not supported at TypeAPI we automatically create an anonymous
            // object and use a reference
            const hash = await this.hash(JSON.stringify(data));
            const anonymousName = 'Object' + hash.substring(0, 8);
            this.anonymousObjects[anonymousName] = data;
            type = 'object';
            ref = anonymousName;
        }
        else if (typeName === 'map' || typeName === 'array') {
            type = typeName;
            ref = this.parseRef(data['additionalProperties']);
        }
        else if (typeName === 'reference') {
            type = 'object';
            ref = data['target'];
        }
        else if (typeName === 'string') {
            type = 'string';
            if (this.isset(data['format'])) {
                format = data['format'];
            }
        }
        else if (typeName === 'integer') {
            type = 'integer';
        }
        else if (typeName === 'number') {
            type = 'number';
        }
        else if (typeName === 'boolean') {
            type = 'boolean';
        }
        else if (typeName === 'any') {
            type = 'any';
        }
        else {
            throw new Error('Could not resolve type: ' + JSON.stringify(data));
        }
        const property = {
            name: name,
            description: data['description'] && typeof data['description'] === 'string' ? data['description'] : '',
            type: type,
        };
        if (this.isset(data['deprecated']) && typeof data['deprecated'] === 'boolean') {
            property.deprecated = data['deprecated'];
        }
        if (this.isset(format) && typeof format === 'string') {
            property.format = format;
        }
        if (ref) {
            property.reference = ref;
        }
        return property;
    }
    parseRef(data) {
        if (this.isset(data.target) && typeof data.target === 'string') {
            return data.target;
        }
        else if (this.isset(data.$ref) && typeof data.$ref === 'string') {
            return this.normalizeRef(data.$ref);
        }
        else if (this.isset(data.$generic)) {
            return 'T';
        }
        else if (this.scalarTypes.includes(data.type)) {
            return data.type;
        }
        else if (data.type === 'array') {
            // at the moment we can not handle array inside maps but we simply return the array type
            return this.parseRef(data.items);
        }
        else {
            throw new Error('Could not resolve ref: ' + JSON.stringify(data));
        }
    }
    isset(value) {
        return typeof value !== 'undefined' && value !== null;
    }
    get(object, keys) {
        for (const key of keys) {
            if (this.isset(object[key])) {
                return object[key];
            }
        }
        return undefined;
    }
    normalizeRef(ref) {
        return ref
            .replace('#/definitions/', '')
            .replace('#/$defs/', '');
    }
    async hash(data) {
        const buffer = await crypto.subtle.digest('SHA-1', new TextEncoder().encode(data));
        return Array.from(new Uint8Array(buffer))
            .map((bytes) => bytes.toString(16).padStart(2, '0'))
            .join('');
    }
}

class JsonSchemaJson extends TypeSchema {
}

class OpenAPIJson extends JsonSchemaJson {
    data = {};
    inlineObjects = {};
    async transform(schema) {
        this.data = JSON.parse(schema);
        const spec = await this.build(this.data);
        this.inlineObjects = {};
        if (this.isset(this.data['paths']) && typeof this.data['paths'] === 'object') {
            for (const [path, value] of Object.entries(this.data['paths'])) {
                if (value !== null && typeof value === 'object') {
                    const operations = await this.transformPath(path, value);
                    operations.forEach((operation) => {
                        spec.operations.push(operation);
                    });
                }
            }
        }
        for (const [key, value] of Object.entries(this.inlineObjects)) {
            try {
                spec.types.push(await this.transformType(key, value));
            }
            catch (error) {
            }
        }
        return spec;
    }
    async transformPath(path, method) {
        const operations = [];
        for (const [methodName, value] of Object.entries(method)) {
            let name = '';
            if (this.isset(value.operationId) && typeof value.operationId === 'string') {
                name = value.operationId.replace('/', '.');
            }
            else {
                name = pascalCase(path.substring(1).replace('/', '.') + '.' + methodName);
            }
            let description = '';
            if (this.isset(value.summary) && typeof value.summary === 'string') {
                description = value.summary;
            }
            else if (this.isset(value.description) && typeof value.description === 'string') {
                description = value.description;
            }
            let args = [];
            if (this.isset(value.parameters) && Array.isArray(value.parameters)) {
                value.parameters.forEach((parameter) => {
                    try {
                        args.push(this.parseArgument(parameter));
                    }
                    catch (error) {
                    }
                });
            }
            let payload = undefined;
            if (this.isset(value.requestBody) && typeof value.requestBody === 'object') {
                payload = await this.parsePayload(value.requestBody);
            }
            let httpCode = 200;
            let result = '';
            let throws = [];
            if (this.isset(value.responses) && typeof value.responses === 'object') {
                for (const [code, resp] of Object.entries(value.responses)) {
                    if (resp === null || typeof resp !== 'object') {
                        continue;
                    }
                    try {
                        let responseCode = parseInt(code);
                        if (responseCode >= 200 && responseCode < 300) {
                            httpCode = responseCode;
                            result = await this.parseResponseRef(resp);
                        }
                        else if (responseCode >= 400 && responseCode < 600) {
                            throws.push({
                                code: responseCode,
                                type: await this.parseResponseRef(resp),
                            });
                        }
                    }
                    catch (error) {
                    }
                }
            }
            let tags = [];
            if (this.isset(value.tags) && Array.isArray(value.tags)) {
                tags = value.tags;
            }
            operations.push({
                name: name,
                description: description,
                httpMethod: methodName.toUpperCase(),
                httpPath: path,
                httpCode: httpCode,
                arguments: args,
                payload: payload,
                payloadShape: undefined,
                throws: throws,
                return: result,
                returnShape: undefined,
                tags: tags
            });
        }
        return operations;
    }
    async parsePayload(requestBody) {
        const payload = await this.parseResponseRef(requestBody);
        if (payload) {
            return payload;
        }
        else {
            return;
        }
    }
    parseArgument(parameter) {
        if (this.isset(parameter['$ref']) && typeof parameter['$ref'] === 'string') {
            const resolved = this.resolve(parameter['$ref']);
            if (!resolved) {
                throw new Error('Could not resolve ref');
            }
            parameter = resolved;
        }
        let type = '';
        if (this.isset(parameter['schema']) && typeof parameter['schema'] === 'object') {
            type = this.parseArgumentSchema(parameter['schema']);
        }
        if (type && (parameter['in'] === 'path' || parameter['in'] === 'query')) {
            return {
                name: parameter['name'],
                in: parameter['in'],
                type: type,
            };
        }
        else {
            throw new Error('Provided an invalid argument');
        }
    }
    parseArgumentSchema(schema) {
        if (this.isset(schema['$ref']) && typeof schema['$ref'] === 'string') {
            const resolved = this.resolve(schema['$ref']);
            if (this.isset(resolved['type']) && typeof resolved['type'] === 'string' && this.scalarTypes.includes(resolved['type'])) {
                return resolved['type'];
            }
        }
        if (this.isset(schema['$ref']) && typeof schema['$ref'] === 'string') {
            return this.normalizeRef(schema['$ref']);
        }
        else if (this.isset(schema['type']) && typeof schema['type'] === 'string') {
            return schema['type'];
        }
        else {
            return '';
        }
    }
    async parseResponseRef(response) {
        if (this.isset(response['$ref']) && typeof response['$ref'] === 'string') {
            response = this.resolve(response['$ref']);
        }
        let schema = this.extractSchema(response);
        if (!schema) {
            return '';
        }
        let ref = schema['$ref'];
        if (typeof ref === 'string') {
            return this.normalizeRef(ref);
        }
        else {
            const hash = await this.hash(JSON.stringify(schema));
            const anonymousName = 'Object' + hash.substring(0, 8);
            this.inlineObjects[anonymousName] = schema;
            return anonymousName;
        }
    }
    extractSchema(response) {
        if (this.isset(response['content']) && typeof response['content'] === 'object') {
            if (this.isset(response['content']['application/json']) && typeof response['content']['application/json'] === 'object') {
                if (this.isset(response['content']['application/json']['schema']) && typeof response['content']['application/json']['schema'] === 'object') {
                    return response['content']['application/json']['schema'];
                }
            }
        }
        return;
    }
    resolve(ref) {
        if (!ref.startsWith('#/')) {
            throw new Error('Can only resolve local refs');
        }
        ref = ref.substring(2);
        let data = this.data;
        ref.split('/').forEach((name) => {
            if (this.isset(data[name])) {
                data = data[name];
            }
            else {
                throw new Error('Could not resolve ' + name);
            }
        });
        return data;
    }
    normalizeRef(ref) {
        return ref
            .replace('#/components/schemas/', '');
    }
}

class TypeAPI extends TypeSchema {
    constructor(typeHubService, resolverService) {
        super(typeHubService, resolverService);
    }
    async transform(schema) {
        const data = JSON.parse(schema);
        const spec = await this.build(data);
        if (this.isset(data.operations) && typeof data.operations === 'object') {
            for (const [key, value] of Object.entries(data.operations)) {
                spec.operations.push(this.transformOperation(key, value));
            }
        }
        return spec;
    }
    transformOperation(name, data) {
        let operation = {
            name: name,
            description: data.description && typeof data.description === 'string' ? data.description : '',
            httpMethod: data.method && typeof data.method === 'string' ? data.method.toUpperCase() : 'GET',
            httpPath: data.path && typeof data.path === 'string' ? data.path : '/',
            httpCode: data.return && data.return.code && typeof data.return.code === 'number' ? data.return.code : 200,
            arguments: [],
            payload: '',
            payloadShape: undefined,
            throws: [],
            return: '',
            returnShape: undefined,
        };
        if (data.arguments) {
            for (const [key, value] of Object.entries(data.arguments)) {
                if (value.in === 'body') {
                    if (value.contentType) {
                        operation.payload = value.contentType;
                        operation.payloadShape = 'mime';
                    }
                    else if (value.schema) {
                        operation.payload = this.parseRef(value.schema);
                        operation.payloadShape = this.parseShape(value.schema);
                    }
                }
                else {
                    operation.arguments.push(this.transformArgument(key, value));
                }
            }
        }
        if (data.throws) {
            for (const [key, value] of Object.entries(data.throws)) {
                operation.throws.push(this.transformThrow(key, value));
            }
        }
        if (data.return) {
            if (data.return.contentType) {
                operation.return = data.return.contentType;
                operation.returnShape = 'mime';
            }
            else if (data.return.schema) {
                operation.return = this.parseRef(data.return.schema);
                operation.returnShape = this.parseShape(data.return.schema);
            }
        }
        if (data.stability && typeof data.stability === 'number') {
            if (data.stability === 0 || data.stability === 1 || data.stability === 2 || data.stability === 3) {
                operation.stability = data.stability;
            }
        }
        if (data.security && Array.isArray(data.security)) {
            operation.security = data.security;
        }
        if (data.authorization && typeof data.authorization === 'boolean') {
            operation.authorization = data.authorization;
        }
        return operation;
    }
    transformArgument(name, data) {
        return {
            name: name,
            in: data.in && typeof data.in === 'string' ? data.in : 'query',
            type: this.parseRef(data.schema),
        };
    }
    transformThrow(name, data) {
        return {
            code: data.code && typeof data.code === 'number' ? data.code : 500,
            type: this.parseRef(data.schema),
        };
    }
    parseShape(data) {
        const typeName = data.type;
        if (this.isset(data.type) && (typeName === 'map' || (data.type === 'object' && this.isset(data.additionalProperties)))) {
            return 'map';
        }
        else if (this.isset(data.type) && (typeName === 'array' || (data.type === 'array' && this.isset(data.items)))) {
            return 'array';
        }
        else {
            return;
        }
    }
}

class OpenAPIYaml extends OpenAPIJson {
    async transform(schema) {
        return super.transform(JSON.stringify(parse(schema)));
    }
}

class JsonSchemaYaml extends JsonSchemaJson {
    async transform(schema) {
        return super.transform(JSON.stringify(parse(schema)));
    }
}

class RawJson {
    typeHubService;
    nestedObjects = [];
    constructor(typeHubService) {
        this.typeHubService = typeHubService;
    }
    async transform(schema) {
        return this.build(JSON.parse(schema));
    }
    async build(data) {
        this.nestedObjects = [];
        const spec = {
            imports: [],
            operations: [],
            types: []
        };
        const root = {
            type: 'object',
            name: await this.buildName(data),
            description: '',
            properties: [],
        };
        for (const [key, value] of Object.entries(data)) {
            const property = await this.transformValue(key, value);
            if (property !== null) {
                root.properties?.push(property);
            }
        }
        spec.types.push(root);
        for (const [key, value] of Object.entries(this.nestedObjects)) {
            spec.types.push(value);
        }
        return spec;
    }
    async transformValue(name, data) {
        if (data === null || data === undefined) {
            return null;
        }
        if (typeof data === 'string') {
            const isDate = new RegExp(/^[0-9]{4}-[0-9]{2}-[0-9]{2}$/, "i");
            const isDateTime = new RegExp(/^[0-9]{4}-[0-9]{2}-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}Z?$/, "i");
            const isTime = new RegExp(/^[0-9]{2}:[0-9]{2}:[0-9]{2}$/, "i");
            let format = null;
            if (isDate.test(data)) {
                format = 'date';
            }
            else if (isDateTime.test(data)) {
                format = 'date-time';
            }
            else if (isTime.test(data)) {
                format = 'time';
            }
            const property = {
                name: name,
                description: '',
                type: 'string',
            };
            if (format !== null) {
                property.format = format;
            }
            return property;
        }
        else if (typeof data === 'number') {
            if (Number.isInteger(data)) {
                return {
                    name: name,
                    description: '',
                    type: 'integer',
                };
            }
            else {
                return {
                    name: name,
                    description: '',
                    type: 'number',
                };
            }
        }
        else if (typeof data === 'boolean') {
            return {
                name: name,
                description: '',
                type: 'boolean',
            };
        }
        else if (Array.isArray(data)) {
            const firstValue = data[0] || null;
            const property = await this.transformValue('', firstValue);
            let ref = '';
            if (property?.reference) {
                ref = property?.reference || '';
            }
            else if (property?.type) {
                ref = property?.type;
            }
            return {
                name: name,
                description: '',
                type: 'array',
                reference: ref
            };
        }
        else if (typeof data === 'object') {
            const typeName = await this.buildName(data);
            const type = {
                type: 'object',
                name: typeName,
                description: '',
                properties: [],
            };
            for (const [key, value] of Object.entries(data)) {
                const property = await this.transformValue(key, value);
                if (property !== null) {
                    type.properties?.push(property);
                }
            }
            this.nestedObjects.push(type);
            return {
                name: name,
                description: '',
                type: 'object',
                reference: typeName
            };
        }
        else {
            return null;
        }
    }
    async buildName(data) {
        const hash = await this.hash(JSON.stringify(data));
        return 'Type' + hash.substring(0, 8);
    }
    hash(data) {
        let hash = 0;
        for (let i = 0; i < data.length; i++) {
            let char = data.charCodeAt(i);
            hash += char;
        }
        return '' + hash;
    }
}

class RawYaml extends RawJson {
    async transform(schema) {
        return super.transform(JSON.stringify(parse(schema)));
    }
}

class TypeHubService {
    httpClient;
    baseUrl = 'https://api.typehub.cloud/';
    client;
    constructor(httpClient) {
        this.httpClient = httpClient;
        this.client = new Client(this.baseUrl);
    }
    async findDocuments(search) {
        return this.client.explore().getAll(0, 16, search);
    }
    async findDocument(user, name) {
        return await this.client.document().get(user, name);
    }
    async export(user, name, version) {
        const response = await this.client.document().export(user, name, {
            version: version,
            format: 'model-typeschema'
        });
        if (!response.href) {
            return;
        }
        const schema = await lastValueFrom(this.httpClient.get(response.href));
        if (typeof schema === 'string') {
            return schema;
        }
        else {
            return JSON.stringify(schema);
        }
    }
    findTags(user, name) {
        return this.client.tag().getAll(user, name);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TypeHubService, deps: [{ token: i1.HttpClient }], target: i0.ɵɵFactoryTarget.Injectable });
    static ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TypeHubService, providedIn: 'root' });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TypeHubService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: i1.HttpClient }] });

class BCLayerService {
    transform(specification) {
        // extract body argument to payload
        specification.operations.forEach((operation, operationIndex) => {
            if (operation.payload) {
                return;
            }
            operation.arguments.forEach((argument, argumentIndex) => {
                if (argument.in === 'body') {
                    operation.payload = argument.type;
                    operation.payloadShape = undefined;
                    specification.operations[operationIndex].arguments?.splice(argumentIndex, 1);
                }
            });
        });
        return specification;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: BCLayerService, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
    static ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: BCLayerService, providedIn: 'root' });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: BCLayerService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }] });

class ResolverService {
    typeHubService;
    httpClient;
    transformer;
    constructor(typeHubService, httpClient) {
        this.typeHubService = typeHubService;
        this.httpClient = httpClient;
        this.transformer = new TypeSchema(typeHubService, this);
    }
    async resolveIncludeTypes(include) {
        if (!include || !include.url) {
            return;
        }
        const url = new URL(include.url);
        let typeApi = null;
        if (url.protocol === 'typehub:') {
            const doc = await this.typeHubService.findDocument(url.username, url.password);
            if (!doc) {
                return;
            }
            typeApi = await this.typeHubService.export(url.username, url.password, url.hostname);
            if (!typeApi) {
                return;
            }
        }
        else if (url.protocol === 'http:' || url.protocol === 'https:') {
            typeApi = await firstValueFrom(this.httpClient.get(url.href, { responseType: 'text' }));
        }
        if (!typeApi) {
            return;
        }
        const spec = await this.transformer.transform(typeApi);
        if (!spec) {
            return;
        }
        return spec.types;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ResolverService, deps: [{ token: TypeHubService }, { token: i1.HttpClient }], target: i0.ɵɵFactoryTarget.Injectable });
    static ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ResolverService, providedIn: 'root' });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ResolverService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: TypeHubService }, { type: i1.HttpClient }] });

class ImportService {
    transformer = {};
    constructor(typeHubService, bcLayerService, resolverService) {
        this.transformer['internal'] = new Internal(bcLayerService);
        this.transformer['typeapi'] = new TypeAPI(typeHubService, resolverService);
        this.transformer['typeschema'] = new TypeSchema(typeHubService, resolverService);
        this.transformer['openapi-json'] = new OpenAPIJson(typeHubService, resolverService);
        this.transformer['openapi-yaml'] = new OpenAPIYaml(typeHubService, resolverService);
        this.transformer['jsonschema-json'] = new JsonSchemaJson(typeHubService, resolverService);
        this.transformer['jsonschema-yaml'] = new JsonSchemaYaml(typeHubService, resolverService);
        this.transformer['raw-json'] = new RawJson(typeHubService);
        this.transformer['raw-yaml'] = new RawYaml(typeHubService);
    }
    async transform(type, schema) {
        if (this.transformer[type]) {
            return this.transformer[type].transform(schema);
        }
        else {
            throw new Error('Provided an invalid type');
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ImportService, deps: [{ token: TypeHubService }, { token: BCLayerService }, { token: ResolverService }], target: i0.ɵɵFactoryTarget.Injectable });
    static ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ImportService, providedIn: 'root' });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ImportService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: TypeHubService }, { type: BCLayerService }, { type: ResolverService }] });

class ArgumentComponent {
    data = [];
    specification;
    dataChange = new EventEmitter();
    result = [];
    newName;
    newIn;
    newType = 'string';
    types = [
        { key: 'string', value: 'String' },
        { key: 'integer', value: 'Integer' },
        { key: 'number', value: 'Number' },
        { key: 'boolean', value: 'Boolean' },
    ];
    ins = [
        { key: 'path', value: 'Path' },
        { key: 'query', value: 'Query' },
        { key: 'header', value: 'Header' },
    ];
    ngOnInit() {
        if (this.data) {
            this.result = [];
            this.data.forEach((entry) => {
                this.result.push(entry);
            });
        }
    }
    add() {
        if (!this.newName || !this.newIn || !this.newType) {
            return;
        }
        this.result.push({
            name: this.newName,
            in: this.newIn,
            type: this.newType,
        });
        this.newName = undefined;
        this.newIn = undefined;
        this.newType = 'string';
    }
    remove(name) {
        this.result = this.result.filter((row) => {
            return row.name !== name;
        });
    }
    changeValue() {
        this.dataChange.emit(this.result);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ArgumentComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: ArgumentComponent, selector: "typeschema-arguments", inputs: { data: "data", specification: "specification" }, outputs: { dataChange: "dataChange" }, ngImport: i0, template: "<div class=\"row mb-3\" *ngFor=\"let row of result;\">\n  <div class=\"col-3\">\n    <input type=\"text\" [id]=\"'name_' + row.name\" [name]=\"'name_' + row.name\" [(ngModel)]=\"row.name\" (ngModelChange)=\"changeValue()\" class=\"form-control\">\n  </div>\n  <div class=\"col-3\">\n    <select [id]=\"'in_' + row.in\" [name]=\"'in_' + row.in\" [(ngModel)]=\"row.in\" class=\"form-select\">\n      <option *ngFor=\"let in_ of ins\" [ngValue]=\"in_.key\">{{in_.value}}</option>\n    </select>\n  </div>\n  <div class=\"col-6\">\n    <div class=\"input-group\">\n      <select [id]=\"'type_' + row.name\" [name]=\"'type_' + row.name\" [(ngModel)]=\"row.type\" class=\"form-select\">\n        <optgroup label=\"scalar\">\n          <option ngValue=\"string\">String</option>\n          <option ngValue=\"integer\">Integer</option>\n          <option ngValue=\"number\">Number</option>\n          <option ngValue=\"boolean\">Boolean</option>\n        </optgroup>\n        <ng-container *ngIf=\"row.in === 'query'\">\n        <optgroup label=\"self\">\n          <option *ngFor=\"let refType of specification.types\" [ngValue]=\"refType.name\">{{ refType.name }}</option>\n        </optgroup>\n        <optgroup label=\"{{ include.alias }}\" *ngFor=\"let include of specification.imports\">\n          <ng-container *ngFor=\"let parentType of include.types\">\n            <option [ngValue]=\"include.alias + ':' + parentType.name\" *ngIf=\"parentType.name != row.name\">{{ parentType.name }}</option>\n          </ng-container>\n        </optgroup>\n        </ng-container>\n      </select>\n      <button class=\"btn btn-outline-secondary\" (click)=\"remove(row.name); changeValue()\"><i class=\"bi bi-x\"></i></button>\n    </div>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"col-3\">\n    <input type=\"text\" id=\"newArgumentName\" name=\"newArgumentName\" [(ngModel)]=\"newName\" (ngModelChange)=\"changeValue()\" class=\"form-control\">\n  </div>\n  <div class=\"col-3\">\n    <select id=\"newArgumentIn\" name=\"newArgumentIn\" [(ngModel)]=\"newIn\" class=\"form-select\">\n      <option *ngFor=\"let in_ of ins\" [ngValue]=\"in_.key\">{{in_.value}}</option>\n    </select>\n  </div>\n  <div class=\"col-6\">\n    <div class=\"input-group\">\n      <select id=\"newArgumentType\" name=\"newArgumentType\" [(ngModel)]=\"newType\" class=\"form-select\">\n        <optgroup label=\"scalar\">\n          <option ngValue=\"string\">String</option>\n          <option ngValue=\"integer\">Integer</option>\n          <option ngValue=\"number\">Number</option>\n          <option ngValue=\"boolean\">Boolean</option>\n        </optgroup>\n      </select>\n      <button class=\"btn btn-outline-secondary\" (click)=\"add(); changeValue()\"><i class=\"bi bi-plus\"></i></button>\n    </div>\n  </div>\n</div>\n", styles: [""], dependencies: [{ kind: "directive", type: i5.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i5.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i2.NgSelectOption, selector: "option", inputs: ["ngValue", "value"] }, { kind: "directive", type: i2.ɵNgSelectMultipleOption, selector: "option", inputs: ["ngValue", "value"] }, { kind: "directive", type: i2.DefaultValueAccessor, selector: "input:not([type=checkbox])[formControlName],textarea[formControlName],input:not([type=checkbox])[formControl],textarea[formControl],input:not([type=checkbox])[ngModel],textarea[ngModel],[ngDefaultControl]" }, { kind: "directive", type: i2.SelectControlValueAccessor, selector: "select:not([multiple])[formControlName],select:not([multiple])[formControl],select:not([multiple])[ngModel]", inputs: ["compareWith"] }, { kind: "directive", type: i2.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { kind: "directive", type: i2.NgModel, selector: "[ngModel]:not([formControlName]):not([formControl])", inputs: ["name", "disabled", "ngModel", "ngModelOptions"], outputs: ["ngModelChange"], exportAs: ["ngModel"] }] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ArgumentComponent, decorators: [{
            type: Component,
            args: [{ selector: 'typeschema-arguments', template: "<div class=\"row mb-3\" *ngFor=\"let row of result;\">\n  <div class=\"col-3\">\n    <input type=\"text\" [id]=\"'name_' + row.name\" [name]=\"'name_' + row.name\" [(ngModel)]=\"row.name\" (ngModelChange)=\"changeValue()\" class=\"form-control\">\n  </div>\n  <div class=\"col-3\">\n    <select [id]=\"'in_' + row.in\" [name]=\"'in_' + row.in\" [(ngModel)]=\"row.in\" class=\"form-select\">\n      <option *ngFor=\"let in_ of ins\" [ngValue]=\"in_.key\">{{in_.value}}</option>\n    </select>\n  </div>\n  <div class=\"col-6\">\n    <div class=\"input-group\">\n      <select [id]=\"'type_' + row.name\" [name]=\"'type_' + row.name\" [(ngModel)]=\"row.type\" class=\"form-select\">\n        <optgroup label=\"scalar\">\n          <option ngValue=\"string\">String</option>\n          <option ngValue=\"integer\">Integer</option>\n          <option ngValue=\"number\">Number</option>\n          <option ngValue=\"boolean\">Boolean</option>\n        </optgroup>\n        <ng-container *ngIf=\"row.in === 'query'\">\n        <optgroup label=\"self\">\n          <option *ngFor=\"let refType of specification.types\" [ngValue]=\"refType.name\">{{ refType.name }}</option>\n        </optgroup>\n        <optgroup label=\"{{ include.alias }}\" *ngFor=\"let include of specification.imports\">\n          <ng-container *ngFor=\"let parentType of include.types\">\n            <option [ngValue]=\"include.alias + ':' + parentType.name\" *ngIf=\"parentType.name != row.name\">{{ parentType.name }}</option>\n          </ng-container>\n        </optgroup>\n        </ng-container>\n      </select>\n      <button class=\"btn btn-outline-secondary\" (click)=\"remove(row.name); changeValue()\"><i class=\"bi bi-x\"></i></button>\n    </div>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"col-3\">\n    <input type=\"text\" id=\"newArgumentName\" name=\"newArgumentName\" [(ngModel)]=\"newName\" (ngModelChange)=\"changeValue()\" class=\"form-control\">\n  </div>\n  <div class=\"col-3\">\n    <select id=\"newArgumentIn\" name=\"newArgumentIn\" [(ngModel)]=\"newIn\" class=\"form-select\">\n      <option *ngFor=\"let in_ of ins\" [ngValue]=\"in_.key\">{{in_.value}}</option>\n    </select>\n  </div>\n  <div class=\"col-6\">\n    <div class=\"input-group\">\n      <select id=\"newArgumentType\" name=\"newArgumentType\" [(ngModel)]=\"newType\" class=\"form-select\">\n        <optgroup label=\"scalar\">\n          <option ngValue=\"string\">String</option>\n          <option ngValue=\"integer\">Integer</option>\n          <option ngValue=\"number\">Number</option>\n          <option ngValue=\"boolean\">Boolean</option>\n        </optgroup>\n      </select>\n      <button class=\"btn btn-outline-secondary\" (click)=\"add(); changeValue()\"><i class=\"bi bi-plus\"></i></button>\n    </div>\n  </div>\n</div>\n" }]
        }], propDecorators: { data: [{
                type: Input
            }], specification: [{
                type: Input
            }], dataChange: [{
                type: Output
            }] } });

class LinkComponent {
    viewportScroller;
    type;
    linkable;
    constructor(viewportScroller) {
        this.viewportScroller = viewportScroller;
    }
    ngOnInit() {
    }
    scrollTo(type) {
        this.viewportScroller.scrollToAnchor('type-' + type);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: LinkComponent, deps: [{ token: i5.ViewportScroller }], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: LinkComponent, selector: "typeschema-link", inputs: { type: "type", linkable: "linkable" }, ngImport: i0, template: "\n<ng-container *ngIf=\"linkable\">\n  <a (click)=\"scrollTo(type)\" role=\"button\" class=\"text-white text-decoration-none\">{{ type }}</a>\n</ng-container>\n<ng-container *ngIf=\"!linkable\">\n  {{ type }}\n</ng-container>\n", styles: [""], dependencies: [{ kind: "directive", type: i5.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: LinkComponent, decorators: [{
            type: Component,
            args: [{ selector: 'typeschema-link', template: "\n<ng-container *ngIf=\"linkable\">\n  <a (click)=\"scrollTo(type)\" role=\"button\" class=\"text-white text-decoration-none\">{{ type }}</a>\n</ng-container>\n<ng-container *ngIf=\"!linkable\">\n  {{ type }}\n</ng-container>\n" }]
        }], ctorParameters: () => [{ type: i5.ViewportScroller }], propDecorators: { type: [{
                type: Input
            }], linkable: [{
                type: Input
            }] } });

class ImportComponent {
    url;
    href = '';
    constructor() { }
    ngOnInit() {
        const url = new URL(this.url);
        if (url.protocol === 'typehub:') {
            this.href = 'https://app.typehub.cloud/d/' + url.username + '/' + url.password + '?version=' + url.hostname;
        }
        else {
            this.href = url.href;
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ImportComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: ImportComponent, selector: "typeschema-import", inputs: { url: "url" }, ngImport: i0, template: "\n<a [href]=\"href\">{{ url }}</a>\n", styles: [""] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ImportComponent, decorators: [{
            type: Component,
            args: [{ selector: 'typeschema-import', template: "\n<a [href]=\"href\">{{ url }}</a>\n" }]
        }], ctorParameters: () => [], propDecorators: { url: [{
                type: Input
            }] } });

class ThrowComponent {
    data = [];
    specification;
    contentTypes;
    dataChange = new EventEmitter();
    result = [];
    newCode;
    newType;
    errorStatusCodes = [
        { key: 0, value: '' },
        { key: 400, value: 'Bad Request' },
        { key: 402, value: 'Payment Required' },
        { key: 403, value: 'Forbidden' },
        { key: 404, value: 'Not Found' },
        { key: 405, value: 'Method Not Allowed' },
        { key: 408, value: 'Request Timeout' },
        { key: 409, value: 'Conflict' },
        { key: 410, value: 'Gone' },
        { key: 412, value: 'Precondition Failed' },
        { key: 417, value: 'Expectation Failed' },
        { key: 422, value: 'Unprocessable Entity' },
        { key: 423, value: 'Locked' },
        { key: 424, value: 'Failed Dependency' },
        { key: 429, value: 'Too Many Requests' },
        { key: 499, value: '4xx' },
        { key: 500, value: 'Internal Server Error' },
        { key: 501, value: 'Not Implemented' },
        { key: 502, value: 'Bad Gateway' },
        { key: 503, value: 'Service Unavailable' },
        { key: 504, value: 'Gateway Timeout' },
        { key: 507, value: 'Insufficient Storage' },
        { key: 508, value: 'Loop Detected' },
        { key: 599, value: '5xx' },
        { key: 999, value: 'Any Error' },
    ];
    ngOnInit() {
        if (this.data) {
            this.result = [];
            this.data.forEach((entry) => {
                this.result.push(entry);
            });
        }
    }
    add() {
        if (!this.newCode || !this.newType) {
            return;
        }
        this.result.push({
            code: this.newCode,
            type: this.newType
        });
        this.newCode = undefined;
        this.newType = undefined;
    }
    removeByIndex(throwIndex) {
        if (!this.result[throwIndex]) {
            return;
        }
        this.result.splice(throwIndex, 1);
    }
    getNotUsedCodes() {
        return this.errorStatusCodes.filter((code) => {
            const selected = this.result.find((row) => {
                return row.code === code.key;
            });
            return !selected;
        });
    }
    changeValue() {
        const result = [];
        this.result.forEach((row) => {
            result.push(row);
        });
        this.dataChange.emit(result);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ThrowComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: ThrowComponent, selector: "typeschema-throws", inputs: { data: "data", specification: "specification", contentTypes: "contentTypes" }, outputs: { dataChange: "dataChange" }, ngImport: i0, template: "<div class=\"row mb-3\" *ngFor=\"let row of result; index as throwIndex\">\n  <div class=\"col-3\">\n    <select [id]=\"'throwCode_' + row.code\" [name]=\"'throwCode_' + row.code\" [(ngModel)]=\"row.code\" [disabled]=\"true\" class=\"form-select\">\n      <option *ngFor=\"let code of errorStatusCodes\" [ngValue]=\"code.key\">{{code.value}}</option>\n    </select>\n  </div>\n  <div class=\"col-3\">\n    <select id=\"operationReturnShape\" name=\"operationReturnShape\" class=\"form-select\" [(ngModel)]=\"row.typeShape\">\n      <option [ngValue]=\"undefined\">Object</option>\n      <option ngValue=\"map\">Map</option>\n      <option ngValue=\"array\">Array</option>\n      <option ngValue=\"mime\">Content-Type</option>\n    </select>\n  </div>\n  <div class=\"col-6\">\n    <div class=\"input-group\">\n      <select *ngIf=\"row.typeShape !== 'mime'\" [id]=\"'throwType_' + row.code\" [name]=\"'throwType_' + row.code\" class=\"form-select\" [(ngModel)]=\"row.type\">\n        <optgroup label=\"self\">\n          <option *ngFor=\"let refType of specification.types\" [ngValue]=\"refType.name\">{{ refType.name }}</option>\n        </optgroup>\n        <optgroup label=\"{{ include.alias }}\" *ngFor=\"let include of specification.imports\">\n          <ng-container *ngFor=\"let parentType of include.types\">\n            <option [ngValue]=\"include.alias + ':' + parentType.name\">{{ parentType.name }}</option>\n          </ng-container>\n        </optgroup>\n      </select>\n      <select *ngIf=\"row.typeShape === 'mime'\" [id]=\"'throwType_' + row.code\" [name]=\"'throwType_' + row.code\" class=\"form-select\" [(ngModel)]=\"row.type\">\n        <option *ngFor=\"let contentType of contentTypes\" [ngValue]=\"contentType.value\">{{ contentType.name }}</option>\n      </select>\n      <button class=\"btn btn-outline-secondary\" (click)=\"removeByIndex(throwIndex); changeValue()\"><i class=\"bi bi-x\"></i></button>\n    </div>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"col-3\">\n    <select id=\"newThrowCode\" name=\"newThrowCode\" [(ngModel)]=\"newCode\" class=\"form-select\">\n      <option *ngFor=\"let code of getNotUsedCodes()\" [ngValue]=\"code.key\">{{code.value}}</option>\n    </select>\n  </div>\n  <div class=\"col-9\">\n    <div class=\"input-group\">\n      <select id=\"newThrowType\" name=\"newThrowType\" class=\"form-select\" [(ngModel)]=\"newType\">\n        <optgroup label=\"self\">\n          <option *ngFor=\"let refType of specification.types\" [ngValue]=\"refType.name\">{{ refType.name }}</option>\n        </optgroup>\n        <optgroup label=\"{{ include.alias }}\" *ngFor=\"let include of specification.imports\">\n          <ng-container *ngFor=\"let parentType of include.types\">\n            <option [ngValue]=\"include.alias + ':' + parentType.name\">{{ parentType.name }}</option>\n          </ng-container>\n        </optgroup>\n      </select>\n      <button class=\"btn btn-outline-secondary\" (click)=\"add(); changeValue()\"><i class=\"bi bi-plus\"></i></button>\n    </div>\n  </div>\n</div>\n", styles: [""], dependencies: [{ kind: "directive", type: i5.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i5.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i2.NgSelectOption, selector: "option", inputs: ["ngValue", "value"] }, { kind: "directive", type: i2.ɵNgSelectMultipleOption, selector: "option", inputs: ["ngValue", "value"] }, { kind: "directive", type: i2.SelectControlValueAccessor, selector: "select:not([multiple])[formControlName],select:not([multiple])[formControl],select:not([multiple])[ngModel]", inputs: ["compareWith"] }, { kind: "directive", type: i2.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { kind: "directive", type: i2.NgModel, selector: "[ngModel]:not([formControlName]):not([formControl])", inputs: ["name", "disabled", "ngModel", "ngModelOptions"], outputs: ["ngModelChange"], exportAs: ["ngModel"] }] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ThrowComponent, decorators: [{
            type: Component,
            args: [{ selector: 'typeschema-throws', template: "<div class=\"row mb-3\" *ngFor=\"let row of result; index as throwIndex\">\n  <div class=\"col-3\">\n    <select [id]=\"'throwCode_' + row.code\" [name]=\"'throwCode_' + row.code\" [(ngModel)]=\"row.code\" [disabled]=\"true\" class=\"form-select\">\n      <option *ngFor=\"let code of errorStatusCodes\" [ngValue]=\"code.key\">{{code.value}}</option>\n    </select>\n  </div>\n  <div class=\"col-3\">\n    <select id=\"operationReturnShape\" name=\"operationReturnShape\" class=\"form-select\" [(ngModel)]=\"row.typeShape\">\n      <option [ngValue]=\"undefined\">Object</option>\n      <option ngValue=\"map\">Map</option>\n      <option ngValue=\"array\">Array</option>\n      <option ngValue=\"mime\">Content-Type</option>\n    </select>\n  </div>\n  <div class=\"col-6\">\n    <div class=\"input-group\">\n      <select *ngIf=\"row.typeShape !== 'mime'\" [id]=\"'throwType_' + row.code\" [name]=\"'throwType_' + row.code\" class=\"form-select\" [(ngModel)]=\"row.type\">\n        <optgroup label=\"self\">\n          <option *ngFor=\"let refType of specification.types\" [ngValue]=\"refType.name\">{{ refType.name }}</option>\n        </optgroup>\n        <optgroup label=\"{{ include.alias }}\" *ngFor=\"let include of specification.imports\">\n          <ng-container *ngFor=\"let parentType of include.types\">\n            <option [ngValue]=\"include.alias + ':' + parentType.name\">{{ parentType.name }}</option>\n          </ng-container>\n        </optgroup>\n      </select>\n      <select *ngIf=\"row.typeShape === 'mime'\" [id]=\"'throwType_' + row.code\" [name]=\"'throwType_' + row.code\" class=\"form-select\" [(ngModel)]=\"row.type\">\n        <option *ngFor=\"let contentType of contentTypes\" [ngValue]=\"contentType.value\">{{ contentType.name }}</option>\n      </select>\n      <button class=\"btn btn-outline-secondary\" (click)=\"removeByIndex(throwIndex); changeValue()\"><i class=\"bi bi-x\"></i></button>\n    </div>\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"col-3\">\n    <select id=\"newThrowCode\" name=\"newThrowCode\" [(ngModel)]=\"newCode\" class=\"form-select\">\n      <option *ngFor=\"let code of getNotUsedCodes()\" [ngValue]=\"code.key\">{{code.value}}</option>\n    </select>\n  </div>\n  <div class=\"col-9\">\n    <div class=\"input-group\">\n      <select id=\"newThrowType\" name=\"newThrowType\" class=\"form-select\" [(ngModel)]=\"newType\">\n        <optgroup label=\"self\">\n          <option *ngFor=\"let refType of specification.types\" [ngValue]=\"refType.name\">{{ refType.name }}</option>\n        </optgroup>\n        <optgroup label=\"{{ include.alias }}\" *ngFor=\"let include of specification.imports\">\n          <ng-container *ngFor=\"let parentType of include.types\">\n            <option [ngValue]=\"include.alias + ':' + parentType.name\">{{ parentType.name }}</option>\n          </ng-container>\n        </optgroup>\n      </select>\n      <button class=\"btn btn-outline-secondary\" (click)=\"add(); changeValue()\"><i class=\"bi bi-plus\"></i></button>\n    </div>\n  </div>\n</div>\n" }]
        }], propDecorators: { data: [{
                type: Input
            }], specification: [{
                type: Input
            }], contentTypes: [{
                type: Input
            }], dataChange: [{
                type: Output
            }] } });

class EditorComponent {
    importService;
    resolverService;
    bcLayerService;
    modalService;
    offCanvasService;
    viewportScroller;
    specification = {
        imports: [],
        operations: [],
        types: [],
    };
    operationEnabled = false;
    importEnabled = true;
    readonly = false;
    id = '';
    objectRef;
    arrayRef;
    mapRef;
    stringRef;
    integerRef;
    numberRef;
    booleanRef;
    save = new EventEmitter();
    change = new EventEmitter();
    operationModalRef;
    typeModalRef;
    operation = {
        name: '',
        description: '',
        httpMethod: 'GET',
        httpPath: '',
        httpCode: 200,
        arguments: [],
        throws: [],
        return: '',
    };
    throw = {
        code: 500,
        type: '',
    };
    type = {
        type: 'object',
        name: '',
        description: '',
        discriminator: '',
        mapping: {},
    };
    children = [];
    generics = [];
    property = {
        name: '',
        description: '',
        type: 'string',
    };
    import = '';
    importType = 'internal';
    export = '';
    selectedType;
    selectedOperation;
    openModal = false;
    loading = false;
    dirty = false;
    response;
    baseUrl;
    security;
    contentTypes = [{
            name: 'application/octet-stream',
            value: 'application/octet-stream'
        }, {
            name: 'application/x-www-form-urlencoded',
            value: 'application/x-www-form-urlencoded'
        }, {
            name: 'application/json',
            value: 'application/json'
        }, {
            name: 'multipart/form-data',
            value: 'multipart/form-data'
        }, {
            name: 'text/plain',
            value: 'text/plain'
        }, {
            name: 'application/xml',
            value: 'application/xml'
        }];
    include = {
        alias: '',
        url: '',
        types: []
    };
    constructor(importService, resolverService, bcLayerService, modalService, offCanvasService, viewportScroller) {
        this.importService = importService;
        this.resolverService = resolverService;
        this.bcLayerService = bcLayerService;
        this.modalService = modalService;
        this.offCanvasService = offCanvasService;
        this.viewportScroller = viewportScroller;
    }
    async ngOnInit() {
        if (!Array.isArray(this.specification.operations)) {
            this.specification.operations = [];
        }
        if (!this.readonly && this.specification.imports.length > 0) {
            for (let i = 0; i < this.specification.imports.length; i++) {
                const include = this.specification.imports[i];
                if (include && !include.types) {
                    this.specification.imports[i].types = await this.resolverService.resolveIncludeTypes(include);
                }
            }
        }
        this.specification = this.bcLayerService.transform(this.specification);
        this.doChange();
    }
    doSave() {
        this.save.emit(this.specification);
        this.dirty = false;
    }
    doChange() {
        this.saveToLocalStorage();
        this.change.emit(this.specification);
    }
    setRoot(typeIndex) {
        this.specification.root = typeIndex;
        this.dirty = true;
        this.doChange();
    }
    orderOperations() {
        this.specification.operations.sort((left, right) => {
            return left.name.localeCompare(right.name);
        });
    }
    openOperation(content) {
        this.openModal = true;
        this.operation = {
            name: '',
            description: '',
            httpMethod: 'GET',
            httpPath: '',
            httpCode: 200,
            arguments: [],
            payload: '',
            payloadShape: undefined,
            throws: [],
            return: '',
            returnShape: undefined,
        };
        this.modalService.open(content, { size: 'lg' }).result.then((result) => {
            const operation = Object.assign({}, this.operation);
            if (!operation.name.match(/^[A-Za-z0-9_.]{1,32}$/)) {
                this.response = {
                    success: false,
                    message: 'Operation name must match the regular expression [A-Za-z0-9_.]{1,32}'
                };
                return;
            }
            this.specification.operations.push(operation);
            this.orderOperations();
            this.dirty = true;
            this.openModal = false;
            this.doChange();
        }, (reason) => {
            this.openModal = false;
        });
    }
    editOperation(content, operationIndex) {
        this.openModal = true;
        this.operation = Object.assign({}, this.specification.operations[operationIndex]);
        this.modalService.open(content, { size: 'lg' }).result.then((result) => {
            const operation = Object.assign({}, this.operation);
            if (!operation.arguments) {
                operation.arguments = [];
            }
            if (!operation.throws) {
                operation.throws = [];
            }
            if (!operation.name.match(/^[A-Za-z0-9_.]{1,32}$/)) {
                this.response = {
                    success: false,
                    message: 'Operation name must match the regular expression [A-Za-z0-9_.]{1,32}'
                };
                return;
            }
            this.specification.operations[operationIndex] = operation;
            this.orderOperations();
            this.dirty = true;
            this.openModal = false;
            this.doChange();
        }, (reason) => {
            this.openModal = false;
        });
    }
    selectOperation(operationIndex) {
        if (this.readonly) {
            return;
        }
        if (this.selectedOperation === operationIndex) {
            this.selectedOperation = undefined;
        }
        else {
            this.selectedOperation = operationIndex;
        }
    }
    orderTypes() {
        this.specification.types.sort((left, right) => {
            return left.name.localeCompare(right.name);
        });
    }
    openType(content) {
        this.openModal = true;
        this.type = {
            type: 'struct',
            name: '',
            description: '',
        };
        this.modalService.open(content, { size: 'lg' }).result.then((result) => {
            const type = Object.assign({}, this.type);
            type.properties = [];
            if (!type.name.match(/^[A-Za-z0-9_]{1,32}$/)) {
                this.response = {
                    success: false,
                    message: 'Type name must match the regular expression [A-Za-z0-9_]{1,32}'
                };
                return;
            }
            this.specification.types.push(type);
            this.orderTypes();
            this.dirty = true;
            this.openModal = false;
            this.doChange();
        }, (reason) => {
            this.openModal = false;
        });
    }
    editType(content, typeIndex) {
        this.openModal = true;
        this.type = Object.assign({}, this.specification.types[typeIndex]);
        this.updateMapping();
        this.updateGenerics();
        this.modalService.open(content, { size: 'lg' }).result.then((result) => {
            const type = Object.assign({}, this.type);
            if (!type.properties) {
                type.properties = [];
            }
            if (!type.name.match(/^[A-Za-z0-9_]{1,32}$/)) {
                this.response = {
                    success: false,
                    message: 'Type name must match the regular expression [A-Za-z0-9_]{1,32}'
                };
                return;
            }
            this.specification.types[typeIndex] = type;
            this.orderTypes();
            this.dirty = true;
            this.openModal = false;
            this.doChange();
        }, (reason) => {
            this.openModal = false;
        });
    }
    deleteTypeMapping(typeIndex, mappingKey) {
        if (!this.specification.types[typeIndex]) {
            return;
        }
        const mapping = this.specification.types[typeIndex].mapping;
        if (!mapping) {
            return;
        }
        if (!mapping[mappingKey]) {
            return;
        }
        delete mapping[mappingKey];
    }
    selectType(typeIndex) {
        if (this.readonly) {
            return;
        }
        if (this.selectedType === typeIndex) {
            this.selectedType = undefined;
        }
        else {
            this.selectedType = typeIndex;
        }
    }
    handleKeyboardEvent(event) {
        if (this.readonly || this.openModal) {
            return;
        }
        if (event.altKey && event.key === 'q' && this.selectedOperation !== undefined) {
            this.editOperation(this.operationModalRef, this.selectedOperation);
        }
        else if (event.altKey && event.key === 'a' && this.selectedOperation !== undefined) {
            this.deleteOperation(this.selectedOperation);
        }
        else if (event.altKey && event.key === 'y') {
            this.openOperation(this.operationModalRef);
        }
        else if (event.altKey && event.key === 'e' && this.selectedType !== undefined) {
            this.editType(this.typeModalRef, this.selectedType);
        }
        else if (event.altKey && event.key === 'd' && this.selectedType !== undefined) {
            this.deleteType(this.selectedType);
        }
        else if (event.altKey && event.key === 'c') {
            this.openType(this.typeModalRef);
        }
    }
    findGenerics(type) {
        if (!type) {
            return [];
        }
        const properties = type.properties;
        if (!properties) {
            return [];
        }
        let result = [];
        for (let i = 0; i < properties.length; i++) {
            const generic = properties[i].generic;
            if (generic) {
                result.push(generic);
            }
        }
        return result;
    }
    findOperationByName(operationName) {
        for (let i = 0; i < this.specification.operations.length; i++) {
            if (this.specification.operations[i].name === operationName) {
                return this.specification.operations[i];
            }
        }
        return null;
    }
    isLinkableType(typeName) {
        if (typeName === 'string' || typeName === 'number' || typeName === 'integer' || typeName === 'boolean' || typeName === 'generic' || typeName === 'any') {
            return false;
        }
        for (let i = 0; i < this.specification.types.length; i++) {
            if (this.specification.types[i].name === typeName) {
                return true;
            }
        }
        return false;
    }
    findTypeByName(typeName) {
        for (let i = 0; i < this.specification.types.length; i++) {
            if (this.specification.types[i].name === typeName) {
                return this.specification.types[i];
            }
        }
        if (typeName.indexOf(':') === -1) {
            return null;
        }
        const parts = typeName.split(':');
        const namespace = parts[0];
        const name = parts[1];
        for (let i = 0; i < this.specification.imports.length; i++) {
            if (this.specification.imports[i].alias !== namespace) {
                continue;
            }
            const types = this.specification.imports[i].types;
            if (!types) {
                continue;
            }
            for (let j = 0; j < types.length; j++) {
                if (types[j].name === name) {
                    return types[j];
                }
            }
        }
        return null;
    }
    findTypesWithParent(parent) {
        if (!parent) {
            return [];
        }
        let result = [];
        for (let i = 0; i < this.specification.types.length; i++) {
            const type = this.specification?.types[i];
            if (type?.parent === parent.name) {
                if (type?.base) {
                    const mapping = type?.mapping;
                    if (mapping) {
                        result = result.concat(Object.keys(mapping));
                    }
                }
                else {
                    result.push(type.name);
                }
            }
        }
        return result;
    }
    updateDiscriminator() {
        this.type.mapping = {};
        this.updateMapping();
    }
    updateMapping() {
        let existing = this.type.mapping || {};
        let mapping = {};
        this.children = this.findTypesWithParent(this.type);
        for (let i = 0; i < this.children.length; i++) {
            mapping[this.children[i]] = this.children[i] in existing ? existing[this.children[i]] : '';
        }
        this.type.mapping = mapping;
    }
    updateGenerics() {
        let existing = this.type.template || {};
        let template = {};
        this.generics = this.findGenerics(this.findTypeByName(this.type.parent || ''));
        for (let i = 0; i < this.generics.length; i++) {
            template[this.generics[i]] = this.generics[i] in existing ? existing[this.generics[i]] : '';
        }
        this.type.template = template;
    }
    updatePropertyReference() {
        if (this.property.reference !== 'generic') {
            this.property.generic = undefined;
        }
    }
    deleteOperation(operationIndex) {
        this.specification.operations.splice(operationIndex, 1);
        this.orderOperations();
        this.dirty = true;
        this.doChange();
    }
    copyOperation(operationIndex) {
        if (!this.specification.operations[operationIndex]) {
            return;
        }
        let newOperation = JSON.parse(JSON.stringify(this.specification.operations[operationIndex]));
        let newName = newOperation.name + '_copy';
        let i = 0;
        while (this.findOperationByName(newName)) {
            i++;
            newName = newOperation.name + '_copy' + i;
        }
        newOperation.name = newName;
        this.specification.operations.push(newOperation);
        this.dirty = true;
        this.doChange();
    }
    upArgument(operationIndex, argumentIndex) {
        const property = this.specification.operations[operationIndex].arguments?.splice(argumentIndex, 1)[0];
        if (!property) {
            return;
        }
        this.specification.operations[operationIndex].arguments?.splice(argumentIndex - 1, 0, property);
        this.dirty = true;
        this.doChange();
    }
    downArgument(operationIndex, argumentIndex) {
        const property = this.specification.operations[operationIndex].arguments?.splice(argumentIndex, 1)[0];
        if (!property) {
            return;
        }
        this.specification.operations[operationIndex].arguments?.splice(argumentIndex + 1, 0, property);
        this.dirty = true;
        this.doChange();
    }
    deleteArgument(operationIndex, argumentIndex) {
        if (!this.specification.operations[operationIndex]) {
            return;
        }
        this.specification.operations[operationIndex].arguments?.splice(argumentIndex, 1);
        this.dirty = true;
        this.doChange();
    }
    upThrow(operationIndex, throwIndex) {
        const property = this.specification.operations[operationIndex].throws?.splice(throwIndex, 1)[0];
        if (!property) {
            return;
        }
        this.specification.operations[operationIndex].throws?.splice(throwIndex - 1, 0, property);
        this.dirty = true;
        this.doChange();
    }
    downThrow(operationIndex, throwIndex) {
        const property = this.specification.operations[operationIndex].throws?.splice(throwIndex, 1)[0];
        if (!property) {
            return;
        }
        this.specification.operations[operationIndex].throws?.splice(throwIndex + 1, 0, property);
        this.dirty = true;
        this.doChange();
    }
    deleteThrow(operationIndex, throwIndex) {
        if (!this.specification.operations[operationIndex]) {
            return;
        }
        this.specification.operations[operationIndex].throws?.splice(throwIndex, 1);
        this.dirty = true;
        this.doChange();
    }
    deleteType(typeIndex) {
        this.specification.types.splice(typeIndex, 1);
        this.orderTypes();
        this.dirty = true;
        this.doChange();
    }
    copyType(typeIndex) {
        if (!this.specification.types[typeIndex]) {
            return;
        }
        let newType = JSON.parse(JSON.stringify(this.specification.types[typeIndex]));
        let newName = newType.name + '_copy';
        let i = 0;
        while (this.findTypeByName(newName)) {
            i++;
            newName = newType.name + '_copy' + i;
        }
        newType.name = newName;
        this.specification.types.push(newType);
        this.dirty = true;
        this.doChange();
    }
    upProperty(typeIndex, propertyIndex) {
        const property = this.specification.types[typeIndex].properties?.splice(propertyIndex, 1)[0];
        if (!property) {
            return;
        }
        this.specification.types[typeIndex].properties?.splice(propertyIndex - 1, 0, property);
        this.dirty = true;
        this.doChange();
    }
    downProperty(typeIndex, propertyIndex) {
        const property = this.specification.types[typeIndex].properties?.splice(propertyIndex, 1)[0];
        if (!property) {
            return;
        }
        this.specification.types[typeIndex].properties?.splice(propertyIndex + 1, 0, property);
        this.dirty = true;
        this.doChange();
    }
    openProperty(content, typeIndex) {
        this.openModal = true;
        this.property = {
            name: '',
            description: '',
            type: 'string',
            metadata: {},
        };
        this.modalService.open(content, { size: 'lg' }).result.then((result) => {
            const property = Object.assign({}, this.property);
            if (!property.name.match(/^[A-Za-z0-9_.$]{1,32}$/)) {
                this.response = {
                    success: false,
                    message: 'Property name must match the regular expression [A-Za-z0-9_.$]{1,32}'
                };
                return;
            }
            this.specification.types[typeIndex].properties?.push(property);
            this.dirty = true;
            this.openModal = false;
            this.doChange();
        }, (reason) => {
            this.openModal = false;
        });
    }
    editProperty(content, typeIndex, propertyIndex) {
        const props = this.specification.types[typeIndex].properties;
        if (!props) {
            return;
        }
        this.openModal = true;
        this.property = Object.assign({}, props[propertyIndex]);
        if (this.property.metadata === undefined) {
            this.property.metadata = {};
        }
        this.modalService.open(content, { size: 'lg' }).result.then((result) => {
            const property = Object.assign({}, this.property);
            if (!property.name.match(/^[A-Za-z0-9_.$]{1,32}$/)) {
                this.response = {
                    success: false,
                    message: 'Property name must match the regular expression [A-Za-z0-9_.$]{1,32}'
                };
                return;
            }
            props[propertyIndex] = property;
            this.dirty = true;
            this.openModal = false;
            this.doChange();
        }, (reason) => {
            this.openModal = false;
        });
    }
    deleteProperty(typeIndex, propertyIndex) {
        if (!this.specification.types[typeIndex]) {
            return;
        }
        this.specification.types[typeIndex].properties?.splice(propertyIndex, 1);
        this.dirty = true;
        this.doChange();
    }
    deleteInclude(includeIndex) {
        this.specification.imports.splice(includeIndex, 1);
        this.dirty = true;
        this.doChange();
    }
    openSettings(content) {
        this.openModal = true;
        this.baseUrl = this.specification.baseUrl || '';
        this.security = this.specification.security || {
            type: 'none'
        };
        this.modalService.open(content, { size: 'lg' }).result.then(async (result) => {
            this.specification.baseUrl = this.baseUrl;
            const availableType = ['httpBasic', 'httpBearer', 'apiKey', 'oauth2'];
            if (this.security?.type && availableType.includes(this.security?.type)) {
                this.specification.security = this.security;
            }
            else {
                this.specification.security = undefined;
            }
            this.dirty = true;
            this.openModal = false;
            this.doChange();
        }, (reason) => {
            this.openModal = false;
        });
    }
    openInclude(content) {
        this.openModal = true;
        this.include = {
            alias: '',
            url: '',
            types: []
        };
        this.modalService.open(content, { size: 'lg' }).result.then(async (result) => {
            const include = this.include;
            include.types = await this.resolverService.resolveIncludeTypes(include);
            this.specification.imports.push(include);
            this.dirty = true;
            this.openModal = false;
            this.doChange();
        }, (reason) => {
            this.openModal = false;
        });
    }
    openImport(content) {
        this.openModal = true;
        this.import = '';
        this.modalService.open(content).result.then(async (result) => {
            this.loading = true;
            const spec = await this.importService.transform(this.importType, this.import);
            this.specification.imports = spec.imports;
            this.specification.operations = spec.operations;
            this.specification.types = spec.types;
            this.specification.root = spec.root;
            this.loading = false;
            this.dirty = true;
            this.openModal = false;
            this.import = '';
            this.doChange();
        }, (reason) => {
            this.openModal = false;
        });
    }
    openExport(content) {
        this.openModal = true;
        this.export = JSON.stringify(this.specification, null, 2);
        this.modalService.open(content).result.then((result) => {
            this.openModal = false;
        }, (reason) => {
            this.openModal = false;
        });
    }
    openToc(content) {
        this.openModal = true;
        this.offCanvasService.open(content, { ariaLabelledBy: 'offcanvas-basic-title' }).result.then((result) => {
            this.openModal = false;
            this.viewportScroller.scrollToAnchor(result);
        }, (reason) => {
            this.openModal = false;
        });
    }
    loadFromLocalStorage() {
        let spec = localStorage.getItem(this.getLocalStorageName());
        if (spec) {
            this.specification = JSON.parse(spec);
            this.dirty = true;
            this.doChange();
        }
    }
    saveToLocalStorage() {
        if (!this.isEmptySpecification()) {
            localStorage.setItem(this.getLocalStorageName(), JSON.stringify(this.specification));
        }
    }
    getLocalStorageName() {
        return 'typeschema_editor_' + this.id;
    }
    isEmptySpecification() {
        return this.specification.imports.length === 0 && this.specification.operations.length === 0 && this.specification.types.length === 0;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: EditorComponent, deps: [{ token: ImportService }, { token: ResolverService }, { token: BCLayerService }, { token: i4.NgbModal }, { token: i4.NgbOffcanvas }, { token: i5.ViewportScroller }], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.13", type: EditorComponent, selector: "typeschema-editor", inputs: { specification: "specification", operationEnabled: "operationEnabled", importEnabled: "importEnabled", readonly: "readonly", id: "id", objectRef: "objectRef", arrayRef: "arrayRef", mapRef: "mapRef", stringRef: "stringRef", integerRef: "integerRef", numberRef: "numberRef", booleanRef: "booleanRef" }, outputs: { save: "save", change: "change" }, host: { listeners: { "document:keydown": "handleKeyboardEvent($event)" } }, viewQueries: [{ propertyName: "operationModalRef", first: true, predicate: ["operationModal"], descendants: true }, { propertyName: "typeModalRef", first: true, predicate: ["typeModal"], descendants: true }], ngImport: i0, template: "\n<ng-container *ngIf=\"!readonly\">\n  <ngb-alert type=\"danger\" *ngIf=\"response && response.success === false\">{{ response.message }}</ngb-alert>\n  <ngb-alert type=\"success\" *ngIf=\"response && response.success === true\">{{ response.message }}</ngb-alert>\n\n  <div class=\"btn-group mb-3 sticky-top\" role=\"group\">\n    <button type=\"button\" class=\"btn btn-primary\" (click)=\"doSave()\" *ngIf=\"save.observed\">Save</button>\n    <button type=\"button\" class=\"btn btn-success\" (click)=\"openOperation(operationModal)\" *ngIf=\"operationEnabled\">New Operation</button>\n    <button type=\"button\" class=\"btn btn-success\" (click)=\"openType(typeModal)\">New Type</button>\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"openSettings(settingsModal)\" *ngIf=\"importEnabled\">Settings</button>\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"openInclude(includeModal)\" *ngIf=\"importEnabled\">Include</button>\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"openImport(importModal)\">Import</button>\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"openExport(exportModal)\">Export</button>\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"openToc(tocOffCanvas)\">TOC</button>\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"loadFromLocalStorage()\">Restore</button>\n  </div>\n</ng-container>\n<ng-container *ngIf=\"readonly\">\n  <div class=\"btn-group mb-3\" role=\"group\">\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"openExport(exportModal)\">Export</button>\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"openToc(tocOffCanvas)\">TOC</button>\n  </div>\n</ng-container>\n\n<div class=\"text-center mt-5\" *ngIf=\"loading\">\n  <div class=\"spinner-border\" role=\"status\">\n    <span class=\"visually-hidden\">Loading...</span>\n  </div>\n</div>\n\n<div class=\"card editor-import mb-3\" *ngIf=\"specification.imports.length > 0\">\n  <div class=\"card-header\">\n    <div class=\"editor-import-headline font-monospace\">\n      <span class=\"badge text-bg-dark\"><span class=\"codicon codicon-cloud-download\"></span></span> Import\n    </div>\n  </div>\n  <div class=\"card-body\">\n    <table class=\"table table table-striped\">\n      <colgroup>\n        <col class=\"w-25\">\n        <col class=\"w-50\">\n        <col class=\"w-25\">\n      </colgroup>\n      <thead>\n      <tr>\n        <th>Alias</th>\n        <th>Url</th>\n        <th></th>\n      </tr>\n      </thead>\n      <tbody>\n      <tr *ngFor=\"let include of specification.imports; index as includeIndex\">\n        <td><span class=\"editor-import-alias\">{{ include.alias }}</span></td>\n        <td><span class=\"editor-import-url\"><typeschema-import [url]=\"include.url\"></typeschema-import></span></td>\n        <td>\n          <div class=\"btn-group float-end\" role=\"group\" *ngIf=\"!readonly\">\n            <a class=\"btn btn-danger\" (click)=\"deleteInclude(includeIndex)\"><span class=\"codicon codicon-close\"></span></a>\n          </div>\n        </td>\n      </tr>\n      </tbody>\n    </table>\n  </div>\n</div>\n\n<ng-container *ngIf=\"operationEnabled\">\n  <div *ngFor=\"let operation of specification.operations; index as operationIndex\" class=\"card editor-type mb-3\" [ngClass]=\"{'border-black': selectedOperation === operationIndex}\" id=\"operation-{{ operation.name }}\" (click)=\"selectOperation(operationIndex)\">\n    <div class=\"card-header\">\n      <div class=\"float-end text-end\">\n        <code>{{ operation.httpMethod }} {{ operation.httpPath }}</code>\n      </div>\n      <div class=\"editor-operation-name font-monospace\"><span class=\"badge text-bg-dark\"><span class=\"codicon codicon-symbol-method\"></span></span> {{ operation.name }}</div>\n      <div class=\"text-secondary editor-operation-description mt-2\" *ngIf=\"operation.description\">{{ operation.description }}</div>\n    </div>\n    <div class=\"card-body\">\n      <table class=\"table table-striped\" *ngIf=\"operation.arguments.length > 0 || operation.payload\">\n        <colgroup>\n          <col class=\"w-50\">\n          <col class=\"w-25\">\n          <col class=\"w-25\">\n        </colgroup>\n        <thead>\n        <tr>\n          <th>Name</th>\n          <th>Type</th>\n          <th></th>\n        </tr>\n        </thead>\n        <tbody>\n        <tr *ngFor=\"let argument of operation.arguments; index as argumentIndex\">\n          <td>\n            <span class=\"editor-operation-argument-name font-monospace\">{{ argument.name }}</span>\n          </td>\n          <td>\n            <span class=\"badge text-bg-primary\"><typeschema-link [type]=\"argument.type\" [linkable]=\"isLinkableType(argument.type)\"></typeschema-link></span>\n            <span class=\"badge text-bg-secondary ms-1\">{{ argument.in }}</span>\n          </td>\n          <td>\n            <div class=\"btn-group float-end\" role=\"group\" *ngIf=\"!readonly\">\n              <button type=\"button\" class=\"btn btn-secondary\" (click)=\"upArgument(operationIndex, argumentIndex)\"><i class=\"bi bi-arrow-up\"></i></button>\n              <button type=\"button\" class=\"btn btn-secondary\" (click)=\"downArgument(operationIndex, argumentIndex)\"><i class=\"bi bi-arrow-down\"></i></button>\n              <button type=\"button\" class=\"btn btn-danger\" (click)=\"deleteArgument(operationIndex, argumentIndex)\"><span class=\"codicon codicon-close\"></span></button>\n            </div>\n          </td>\n        </tr>\n        <tr *ngIf=\"operation.payload\">\n          <td>\n            <span class=\"editor-operation-argument-name font-monospace\">payload</span>\n          </td>\n          <td>\n            <span class=\"badge text-bg-primary\">\n              <ng-container *ngIf=\"operation.payloadShape === 'array'\">Array&lt;<typeschema-link [type]=\"operation.payload\" [linkable]=\"isLinkableType(operation.payload)\"></typeschema-link>&gt;</ng-container>\n              <ng-container *ngIf=\"operation.payloadShape === 'map'\">Map&lt;string, <typeschema-link [type]=\"operation.payload\" [linkable]=\"isLinkableType(operation.payload)\"></typeschema-link>&gt;</ng-container>\n              <ng-container *ngIf=\"operation.payloadShape === 'mime'\">{{ operation.payload }}</ng-container>\n              <ng-container *ngIf=\"operation.payloadShape !== 'array' && operation.payloadShape !== 'map' && operation.payloadShape !== 'mime'\"><typeschema-link [type]=\"operation.payload\" [linkable]=\"isLinkableType(operation.payload)\"></typeschema-link></ng-container>\n            </span>\n            <span class=\"badge text-bg-secondary ms-1\">body</span>\n          </td>\n          <td></td>\n        </tr>\n        </tbody>\n      </table>\n      <table class=\"table table-striped\" *ngIf=\"operation.httpCode || operation.throws.length > 0\">\n        <colgroup>\n          <col class=\"w-50\">\n          <col class=\"w-25\">\n          <col class=\"w-25\">\n        </colgroup>\n        <thead>\n        <tr>\n          <th>Code</th>\n          <th>Type</th>\n          <th></th>\n        </tr>\n        </thead>\n        <tbody>\n        <tr *ngIf=\"operation.httpCode === 204\">\n          <td><span class=\"badge text-bg-success\">{{ operation.httpCode }}</span></td>\n          <td><span class=\"badge text-bg-secondary\">-</span></td>\n          <td></td>\n        </tr>\n        <tr *ngIf=\"operation.httpCode && operation.return\">\n          <td><span class=\"badge text-bg-success\">{{ operation.httpCode }}</span></td>\n          <td>\n            <span class=\"badge text-bg-primary\">\n              <ng-container *ngIf=\"operation.returnShape === 'array'\">Array&lt;<typeschema-link [type]=\"operation.return\" [linkable]=\"isLinkableType(operation.return)\"></typeschema-link>&gt;</ng-container>\n              <ng-container *ngIf=\"operation.returnShape === 'map'\">Map&lt;string, <typeschema-link [type]=\"operation.return\" [linkable]=\"isLinkableType(operation.return)\"></typeschema-link>&gt;</ng-container>\n              <ng-container *ngIf=\"operation.returnShape === 'mime'\">{{ operation.return }}</ng-container>\n              <ng-container *ngIf=\"operation.returnShape !== 'array' && operation.returnShape !== 'map' && operation.returnShape !== 'mime'\"><typeschema-link [type]=\"operation.return\" [linkable]=\"isLinkableType(operation.return)\"></typeschema-link></ng-container>\n            </span>\n          </td>\n          <td></td>\n        </tr>\n        <tr *ngFor=\"let throw_ of operation.throws; index as throwIndex\">\n          <td><span class=\"badge text-bg-danger\">{{ throw_.code }}</span></td>\n          <td>\n            <span class=\"badge text-bg-primary\">\n              <ng-container *ngIf=\"throw_.typeShape === 'array'\">Array&lt;<typeschema-link [type]=\"throw_.type\" [linkable]=\"isLinkableType(throw_.type)\"></typeschema-link>&gt;</ng-container>\n              <ng-container *ngIf=\"throw_.typeShape === 'map'\">Map&lt;string, <typeschema-link [type]=\"throw_.type\" [linkable]=\"isLinkableType(throw_.type)\"></typeschema-link>&gt;</ng-container>\n              <ng-container *ngIf=\"throw_.typeShape === 'mime'\">{{ throw_.type }}</ng-container>\n              <ng-container *ngIf=\"throw_.typeShape !== 'array' && throw_.typeShape !== 'map' && throw_.typeShape !== 'mime'\"><typeschema-link [type]=\"throw_.type\" [linkable]=\"isLinkableType(throw_.type)\"></typeschema-link></ng-container>\n            </span>\n          </td>\n          <td>\n            <div class=\"btn-group float-end\" role=\"group\" *ngIf=\"!readonly\">\n              <button type=\"button\" class=\"btn btn-secondary\" (click)=\"upThrow(operationIndex, throwIndex)\"><i class=\"bi bi-arrow-up\"></i></button>\n              <button type=\"button\" class=\"btn btn-secondary\" (click)=\"downThrow(operationIndex, throwIndex)\"><i class=\"bi bi-arrow-down\"></i></button>\n              <button type=\"button\" class=\"btn btn-danger\" (click)=\"deleteThrow(operationIndex, throwIndex)\"><span class=\"codicon codicon-close\"></span></button>\n            </div>\n          </td>\n        </tr>\n        </tbody>\n      </table>\n    </div>\n    <div class=\"card-footer\" *ngIf=\"!readonly\">\n      <div class=\"btn-group\" role=\"group\">\n        <button type=\"button\" class=\"btn btn-primary\" (click)=\"editOperation(operationModal, operationIndex)\"><span class=\"codicon codicon-edit\"></span></button>\n        <button type=\"button\" class=\"btn btn-danger\" (click)=\"deleteOperation(operationIndex)\"><span class=\"codicon codicon-close\"></span></button>\n        <button type=\"button\" class=\"btn btn-secondary\" (click)=\"copyOperation(operationIndex)\"><span class=\"codicon codicon-copy\"></span></button>\n      </div>\n    </div>\n  </div>\n</ng-container>\n\n<div *ngFor=\"let type of specification.types; index as typeIndex\" class=\"card editor-type mb-3\" [ngClass]=\"{'border-primary': specification.root === typeIndex, 'border-black': selectedType === typeIndex}\" id=\"type-{{ type.name }}\" (click)=\"selectType(typeIndex)\">\n  <div class=\"card-header\">\n    <div class=\"editor-type-name font-monospace\"><span class=\"badge text-bg-dark\">\n      <span class=\"codicon codicon-symbol-structure\"></span></span> {{ type.name }}\n      <span *ngIf=\"type.parent\" class=\"float-end\"><span class=\"badge text-bg-primary\"><typeschema-link [type]=\"type.parent\" [linkable]=\"isLinkableType(type.parent)\"></typeschema-link></span></span>\n    </div>\n    <div class=\"text-secondary editor-type-description mt-2\" *ngIf=\"type.description\">{{ type.description }}</div>\n  </div>\n  <div class=\"card-body\">\n    <div [ngSwitch]=\"type.type\">\n      <ng-container *ngSwitchCase=\"'map'\">\n        <table class=\"table table table-striped editor-type-properties\">\n          <colgroup>\n            <col class=\"w-50\">\n            <col class=\"w-25\">\n            <col class=\"w-25\">\n          </colgroup>\n          <thead>\n          <tr>\n            <th>Name</th>\n            <th>Type</th>\n            <th></th>\n          </tr>\n          </thead>\n          <tbody>\n          <tr>\n            <td><div class=\"editor-type-property-name font-monospace\">*</div></td>\n            <td><span class=\"badge text-bg-primary\" *ngIf=\"type.reference\"><typeschema-link [type]=\"type.reference\" [linkable]=\"isLinkableType(type.reference)\"></typeschema-link></span></td>\n            <td></td>\n          </tr>\n          </tbody>\n        </table>\n      </ng-container>\n      <ng-container *ngSwitchDefault>\n        <div *ngIf=\"type.discriminator\">\n          <div>Discriminator: <span class=\"badge text-bg-primary\">{{ type.discriminator }}</span></div>\n          <table class=\"table table table-striped editor-type-properties\" *ngIf=\"type.mapping\">\n            <colgroup>\n              <col class=\"w-50\">\n              <col class=\"w-25\">\n              <col class=\"w-25\">\n            </colgroup>\n            <thead>\n            <tr>\n              <th>Value</th>\n              <th>Type</th>\n              <th></th>\n            </tr>\n            </thead>\n            <tbody>\n            <tr *ngFor=\"let mapping of type.mapping|keyvalue\">\n              <td><code>{{ mapping.value }}</code></td>\n              <td>\n                <div class=\"editor-type-property-name\">\n                  <span class=\"badge text-bg-primary\">\n                    <typeschema-link [type]=\"mapping.key\" [linkable]=\"isLinkableType(mapping.key)\"></typeschema-link>\n                  </span>\n                </div>\n              </td>\n              <td>\n                <div class=\"btn-group float-end\" role=\"group\" *ngIf=\"!readonly\">\n                  <button type=\"button\" class=\"btn btn-danger\" (click)=\"deleteTypeMapping(typeIndex, mapping.key)\"><span class=\"codicon codicon-close\"></span></button>\n                </div>\n              </td>\n            </tr>\n            </tbody>\n          </table>\n        </div>\n        <table class=\"table table table-striped editor-type-properties\">\n          <colgroup>\n            <col class=\"w-50\">\n            <col class=\"w-25\">\n            <col class=\"w-25\">\n          </colgroup>\n          <thead>\n          <tr>\n            <th>Name</th>\n            <th>Type</th>\n            <th></th>\n          </tr>\n          </thead>\n          <tbody>\n          <tr *ngFor=\"let property of type.properties; index as propertyIndex\">\n            <td>\n              <div class=\"editor-type-property-name font-monospace\">{{ property.name }}</div>\n              <div class=\"editor-type-property-description text-secondary mt-2\" *ngIf=\"property.description\">{{ property.description }}</div>\n            </td>\n            <td>\n              <span class=\"badge text-bg-primary\">{{ property.type }}</span>\n              <span class=\"badge text-bg-secondary ms-1\" *ngIf=\"property.reference\">\n                <typeschema-link [type]=\"property.reference\" [linkable]=\"isLinkableType(property.reference)\"></typeschema-link>\n              </span>\n              <span class=\"badge text-bg-secondary ms-1\" *ngIf=\"property.generic\">{{ property.generic }}</span>\n              <span class=\"badge text-bg-secondary ms-1\" *ngIf=\"property.format\">{{ property.format }}</span>\n            </td>\n            <td>\n              <div class=\"btn-group float-end\" role=\"group\" *ngIf=\"!readonly\">\n                <button type=\"button\" class=\"btn btn-secondary\" (click)=\"upProperty(typeIndex, propertyIndex)\"><i class=\"bi bi-arrow-up\"></i></button>\n                <button type=\"button\" class=\"btn btn-secondary\" (click)=\"downProperty(typeIndex, propertyIndex)\"><i class=\"bi bi-arrow-down\"></i></button>\n                <button type=\"button\" class=\"btn btn-primary\" (click)=\"editProperty(propertyModal, typeIndex, propertyIndex)\"><span class=\"codicon codicon-edit\"></span></button>\n                <button type=\"button\" class=\"btn btn-danger\" (click)=\"deleteProperty(typeIndex, propertyIndex)\"><span class=\"codicon codicon-close\"></span></button>\n              </div>\n            </td>\n          </tr>\n          </tbody>\n        </table>\n        <div class=\"btn-group\" role=\"group\" *ngIf=\"!readonly\">\n          <a class=\"btn btn-success\" (click)=\"openProperty(propertyModal, typeIndex)\">New Property</a>\n        </div>\n      </ng-container>\n    </div>\n  </div>\n  <div class=\"card-footer\" *ngIf=\"!readonly\">\n    <div class=\"btn-group\" role=\"group\">\n      <button type=\"button\" class=\"btn btn-primary\" (click)=\"editType(typeModal, typeIndex)\"><span class=\"codicon codicon-edit\"></span></button>\n      <button type=\"button\" class=\"btn btn-danger\" (click)=\"deleteType(typeIndex)\"><span class=\"codicon codicon-close\"></span></button>\n      <button type=\"button\" class=\"btn btn-secondary\" (click)=\"copyType(typeIndex)\"><span class=\"codicon codicon-copy\"></span></button>\n      <button *ngIf=\"specification.root !== typeIndex\" type=\"button\" class=\"btn btn-secondary\" (click)=\"setRoot(typeIndex)\">Primary</button>\n    </div>\n  </div>\n</div>\n\n<ng-template #operationModal let-modal>\n  <div class=\"modal-header\">\n    <h4 class=\"modal-title\">Operation</h4>\n    <button type=\"button\" class=\"btn-close\" aria-label=\"Close\" (click)=\"modal.dismiss()\"></button>\n  </div>\n  <div class=\"modal-body\">\n    <div class=\"mb-3 row\">\n      <label for=\"operationName\" class=\"col-sm-2 col-form-label fw-bold\">Name</label>\n      <div class=\"col-sm-10\">\n        <input id=\"operationName\" name=\"operationName\" class=\"form-control\" placeholder=\"my.operation.execute\" [(ngModel)]=\"operation.name\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"operationDescription\" class=\"col-sm-2 col-form-label fw-bold\">Description</label>\n      <div class=\"col-sm-10\">\n        <input id=\"operationDescription\" name=\"operationDescription\" class=\"form-control\" [(ngModel)]=\"operation.description\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"operationHttpMethod\" class=\"col-sm-2 col-form-label fw-bold\">HTTP Method</label>\n      <div class=\"col-sm-10\">\n        <select id=\"operationHttpMethod\" name=\"operationHttpMethod\" class=\"form-select\" [(ngModel)]=\"operation.httpMethod\">\n          <option ngValue=\"GET\">GET</option>\n          <option ngValue=\"POST\">POST</option>\n          <option ngValue=\"PUT\">PUT</option>\n          <option ngValue=\"PATCH\">PATCH</option>\n          <option ngValue=\"DELETE\">DELETE</option>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"operationHttpPath\" class=\"col-sm-2 col-form-label fw-bold\">HTTP Path</label>\n      <div class=\"col-sm-10\">\n        <input id=\"operationHttpPath\" name=\"operationHttpPath\" class=\"form-control\" placeholder=\"/my/endpoint\" [(ngModel)]=\"operation.httpPath\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"operationHttpCode\" class=\"col-sm-2 col-form-label fw-bold\">HTTP Code</label>\n      <div class=\"col-sm-10\">\n        <select id=\"operationHttpCode\" name=\"operationHttpCode\" class=\"form-select\" [(ngModel)]=\"operation.httpCode\">\n          <option [ngValue]=\"200\">200 OK</option>\n          <option [ngValue]=\"201\">201 Created</option>\n          <option [ngValue]=\"202\">202 Accepted</option>\n          <option [ngValue]=\"204\">204 No Content</option>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"operationArguments\" class=\"col-sm-2 col-form-label fw-bold\">Arguments</label>\n      <div class=\"col-sm-10\">\n        <typeschema-arguments [data]=\"operation.arguments\" [specification]=\"specification\" (dataChange)=\"operation.arguments = $event\"></typeschema-arguments>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"operation.httpMethod === 'POST' || operation.httpMethod === 'PUT' || operation.httpMethod === 'PATCH'\">\n      <label for=\"operationPayload\" class=\"col-sm-2 col-form-label fw-bold\">Payload</label>\n      <div class=\"col-sm-10\">\n        <div class=\"row mb-3\">\n          <div class=\"col-3\">\n            <select id=\"operationPayloadShape\" name=\"operationPayloadShape\" class=\"form-select\" [(ngModel)]=\"operation.payloadShape\">\n              <option [ngValue]=\"undefined\">Object</option>\n              <option ngValue=\"map\">Map</option>\n              <option ngValue=\"array\">Array</option>\n              <option ngValue=\"mime\">Content-Type</option>\n            </select>\n          </div>\n          <div class=\"col-9\">\n            <select *ngIf=\"operation.payloadShape !== 'mime'\" id=\"operationPayload\" name=\"operationPayload\" class=\"form-select\" [(ngModel)]=\"operation.payload\">\n              <optgroup label=\"self\">\n                <option *ngFor=\"let refType of specification.types\" [ngValue]=\"refType.name\">{{ refType.name }}</option>\n              </optgroup>\n              <optgroup label=\"{{ include.alias }}\" *ngFor=\"let include of specification.imports\">\n                <ng-container *ngFor=\"let parentType of include.types\">\n                  <option [ngValue]=\"include.alias + ':' + parentType.name\" *ngIf=\"parentType.name != type.name\">{{ parentType.name }}</option>\n                </ng-container>\n              </optgroup>\n            </select>\n            <select *ngIf=\"operation.payloadShape === 'mime'\" id=\"operationPayload\" name=\"operationPayload\" class=\"form-select\" [(ngModel)]=\"operation.payload\">\n              <option *ngFor=\"let contentType of contentTypes\" [ngValue]=\"contentType.value\">{{ contentType.name }}</option>\n            </select>\n          </div>\n        </div>\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"operationThrows\" class=\"col-sm-2 col-form-label fw-bold\">Throws</label>\n      <div class=\"col-sm-10\">\n        <typeschema-throws [data]=\"operation.throws\" [specification]=\"specification\" [contentTypes]=\"contentTypes\" (dataChange)=\"operation.throws = $event\"></typeschema-throws>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"operation.httpCode !== 204\">\n      <label for=\"operationReturn\" class=\"col-sm-2 col-form-label fw-bold\">Return</label>\n      <div class=\"col-sm-10\">\n        <div class=\"row mb-3\">\n          <div class=\"col-3\">\n            <select id=\"operationReturnShape\" name=\"operationReturnShape\" class=\"form-select\" [(ngModel)]=\"operation.returnShape\">\n              <option [ngValue]=\"undefined\">Object</option>\n              <option ngValue=\"map\">Map</option>\n              <option ngValue=\"array\">Array</option>\n              <option ngValue=\"mime\">Content-Type</option>\n            </select>\n          </div>\n          <div class=\"col-9\">\n            <select *ngIf=\"operation.returnShape !== 'mime'\" id=\"operationReturn\" name=\"operationReturn\" class=\"form-select\" [(ngModel)]=\"operation.return\">\n              <optgroup label=\"self\">\n                <option *ngFor=\"let refType of specification.types\" [ngValue]=\"refType.name\">{{ refType.name }}</option>\n              </optgroup>\n              <optgroup label=\"{{ include.alias }}\" *ngFor=\"let include of specification.imports\">\n                <ng-container *ngFor=\"let parentType of include.types\">\n                  <option [ngValue]=\"include.alias + ':' + parentType.name\" *ngIf=\"parentType.name != type.name\">{{ parentType.name }}</option>\n                </ng-container>\n              </optgroup>\n            </select>\n            <select *ngIf=\"operation.returnShape === 'mime'\" id=\"operationReturn\" name=\"operationReturn\" class=\"form-select\" [(ngModel)]=\"operation.return\">\n              <option *ngFor=\"let contentType of contentTypes\" [ngValue]=\"contentType.value\">{{ contentType.name }}</option>\n            </select>\n          </div>\n        </div>\n      </div>\n    </div>\n  </div>\n  <div class=\"modal-footer\">\n    <button type=\"button\" class=\"btn btn-outline-dark\" (click)=\"modal.close()\">Save</button>\n  </div>\n</ng-template>\n\n<ng-template #typeModal let-modal>\n  <div class=\"modal-header\">\n    <h4 class=\"modal-title\">Type</h4>\n    <button type=\"button\" class=\"btn-close\" aria-label=\"Close\" (click)=\"modal.dismiss()\"></button>\n  </div>\n  <div class=\"modal-body\">\n    <div class=\"mb-3 row\">\n      <label for=\"typeType\" class=\"col-sm-2 col-form-label fw-bold\">Type</label>\n      <div class=\"col-sm-10\">\n        <select id=\"typeType\" name=\"typeType\" class=\"form-select\" [(ngModel)]=\"type.type\">\n          <option ngValue=\"struct\">Struct</option>\n          <option ngValue=\"map\">Map</option>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"typeName\" class=\"col-sm-2 col-form-label fw-bold\">Name</label>\n      <div class=\"col-sm-10\">\n        <input id=\"typeName\" name=\"typeName\" class=\"form-control\" [(ngModel)]=\"type.name\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"typeDescription\" class=\"col-sm-2 col-form-label fw-bold\">Description</label>\n      <div class=\"col-sm-10\">\n        <input id=\"typeDescription\" name=\"typeDescription\" class=\"form-control\" [(ngModel)]=\"type.description\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"type.type === 'struct'\">\n      <label for=\"typeBase\" class=\"col-sm-2 col-form-label fw-bold\">Abstract</label>\n      <div class=\"col-sm-10\">\n        <div class=\"form-check form-switch mt-2\">\n          <input class=\"form-check-input\" type=\"checkbox\" role=\"switch\" id=\"typeBase\" [(ngModel)]=\"type.base\">\n        </div>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"type.type === 'struct'\">\n      <label for=\"typeParent\" class=\"col-sm-2 col-form-label fw-bold\">Parent</label>\n      <div class=\"col-sm-10\">\n        <select id=\"typeParent\" name=\"typeParent\" class=\"form-select\" [(ngModel)]=\"type.parent\" (ngModelChange)=\"updateGenerics()\">\n          <option [ngValue]=\"undefined\">No parent</option>\n          <optgroup label=\"Self\">\n            <ng-container *ngFor=\"let parentType of specification.types\">\n              <option [ngValue]=\"parentType.name\" *ngIf=\"parentType.name != type.name\">{{ parentType.name }}</option>\n            </ng-container>\n          </optgroup>\n          <optgroup label=\"{{include.alias}}\" *ngFor=\"let include of specification.imports\">\n            <ng-container *ngFor=\"let parentType of include.types\">\n              <option [ngValue]=\"include.alias + ':' + parentType.name\">{{ parentType.name }}</option>\n            </ng-container>\n          </optgroup>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"type.type === 'struct' && type.base\">\n      <label for=\"typeDiscriminator\" class=\"col-sm-2 col-form-label fw-bold\">Discriminator</label>\n      <div class=\"col-sm-10\">\n        <select id=\"typeDiscriminator\" name=\"typeDiscriminator\" class=\"form-select\" [(ngModel)]=\"type.discriminator\" (ngModelChange)=\"updateDiscriminator()\">\n          <option [ngValue]=\"undefined\">No discriminator</option>\n          <ng-container *ngFor=\"let property of type.properties\">\n            <option [ngValue]=\"property.name\">{{ property.name }}</option>\n          </ng-container>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"type.type === 'struct' && type.discriminator && type.mapping && (type.mapping | keyvalue)?.length\">\n      <label class=\"col-sm-2 col-form-label fw-bold\">Mapping</label>\n      <div class=\"col-sm-10\">\n        <div *ngFor=\"let child of children\">\n          <div class=\"input-group mb-3\">\n            <input id=\"typeMapping-{{ child }}\" name=\"typeMapping-{{ child }}\" type=\"text\" class=\"form-control\" [(ngModel)]=\"type.mapping[child]\" placeholder=\"Value of the discriminator property\">\n            <span class=\"input-group-text\">{{ child }}</span>\n          </div>\n        </div>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"type.type === 'struct' && type.template && (type.template | keyvalue)?.length\">\n      <label class=\"col-sm-2 col-form-label fw-bold\">Template</label>\n      <div class=\"col-sm-10\">\n        <div *ngFor=\"let generic of generics\">\n          <div class=\"input-group mb-3\">\n            <select id=\"typeTemplate-{{ generic }}\" name=\"typeTemplate-{{ generic }}\" class=\"form-select\" [(ngModel)]=\"type.template[generic]\">\n              <option>No type</option>\n              <ng-container *ngFor=\"let type of specification.types\">\n                <option [ngValue]=\"type.name\">{{ type.name }}</option>\n              </ng-container>\n            </select>\n            <span class=\"input-group-text\">{{ generic }}</span>\n          </div>\n        </div>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"type.type === 'map'\">\n      <label for=\"typeReference\" class=\"col-sm-2 col-form-label fw-bold\">Reference</label>\n      <div class=\"col-sm-10\">\n        <select id=\"typeReference\" name=\"typeReference\" class=\"form-select\" [(ngModel)]=\"type.reference\">\n          <optgroup label=\"scalar\">\n            <option ngValue=\"string\">String</option>\n            <option ngValue=\"integer\">Integer</option>\n            <option ngValue=\"number\">Number</option>\n            <option ngValue=\"boolean\">Boolean</option>\n            <option ngValue=\"any\">Any</option>\n            <option ngValue=\"generic\">Generic</option>\n          </optgroup>\n          <optgroup label=\"self\">\n            <option *ngFor=\"let refType of specification.types\" [ngValue]=\"refType.name\">{{ refType.name }}</option>\n          </optgroup>\n          <optgroup label=\"{{ include.alias }}\" *ngFor=\"let include of specification.imports\">\n            <ng-container *ngFor=\"let parentType of include.types\">\n              <option [ngValue]=\"include.alias + ':' + parentType.name\" *ngIf=\"parentType.name != type.name\">{{ parentType.name }}</option>\n            </ng-container>\n          </optgroup>\n        </select>\n      </div>\n    </div>\n  </div>\n  <div class=\"modal-footer\">\n    <button type=\"button\" class=\"btn btn-outline-dark\" (click)=\"modal.close()\">Save</button>\n  </div>\n</ng-template>\n\n<ng-template #propertyModal let-modal>\n  <div class=\"modal-header\">\n    <h4 class=\"modal-title\">Property</h4>\n    <button type=\"button\" class=\"btn-close\" aria-label=\"Close\" (click)=\"modal.dismiss()\"></button>\n  </div>\n  <div class=\"modal-body\">\n    <div class=\"mb-3 row\">\n      <label for=\"propertyName\" class=\"col-sm-2 col-form-label fw-bold\">Name</label>\n      <div class=\"col-sm-10\">\n        <input id=\"propertyName\" name=\"propertyName\" class=\"form-control\" [(ngModel)]=\"property.name\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"propertyDescription\" class=\"col-sm-2 col-form-label fw-bold\">Description</label>\n      <div class=\"col-sm-10\">\n        <input id=\"propertyDescription\" name=\"propertyDescription\" class=\"form-control\" [(ngModel)]=\"property.description\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"propertyType\" class=\"col-sm-2 col-form-label fw-bold\">Type</label>\n      <div class=\"col-sm-10\">\n        <select id=\"propertyType\" name=\"propertyType\" class=\"form-select\" [(ngModel)]=\"property.type\">\n          <optgroup label=\"Scalar\">\n            <option ngValue=\"string\">String</option>\n            <option ngValue=\"integer\">Integer</option>\n            <option ngValue=\"number\">Number</option>\n            <option ngValue=\"boolean\">Boolean</option>\n          </optgroup>\n          <optgroup label=\"Complex\">\n            <option ngValue=\"object\">Object</option>\n            <option ngValue=\"map\">Map</option>\n            <option ngValue=\"array\">Array</option>\n            <option ngValue=\"any\">Any</option>\n            <option ngValue=\"generic\">Generic</option>\n          </optgroup>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"property.type == 'string'\">\n      <label for=\"propertyFormat\" class=\"col-sm-2 col-form-label fw-bold\">Format</label>\n      <div class=\"col-sm-10\">\n        <select id=\"propertyFormat\" name=\"propertyFormat\" class=\"form-select\" [(ngModel)]=\"property.format\">\n          <option [ngValue]=\"undefined\">No Format</option>\n          <option *ngIf=\"property.type == 'string'\" ngValue=\"date\">Date</option>\n          <option *ngIf=\"property.type == 'string'\" ngValue=\"date-time\">Date-Time</option>\n          <option *ngIf=\"property.type == 'string'\" ngValue=\"time\">Time</option>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"property.type == 'generic'\">\n      <label for=\"propertyGeneric\" class=\"col-sm-2 col-form-label fw-bold\">Generic</label>\n      <div class=\"col-sm-10\">\n        <input id=\"propertyGeneric\" name=\"propertyGeneric\" class=\"form-control\" [(ngModel)]=\"property.generic\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"property.type == 'object' || property.type == 'array' || property.type == 'map'\">\n      <label for=\"propertyReference\" class=\"col-sm-2 col-form-label fw-bold\">Reference</label>\n      <div class=\"col-sm-10\">\n        <select id=\"propertyReference\" name=\"propertyReference\" class=\"form-select\" [(ngModel)]=\"property.reference\" (ngModelChange)=\"updatePropertyReference()\">\n          <optgroup label=\"scalar\" *ngIf=\"property.type != 'object'\">\n            <option ngValue=\"string\">String</option>\n            <option ngValue=\"integer\">Integer</option>\n            <option ngValue=\"number\">Number</option>\n            <option ngValue=\"boolean\">Boolean</option>\n            <option ngValue=\"any\">Any</option>\n            <option ngValue=\"generic\">Generic</option>\n          </optgroup>\n          <optgroup label=\"self\">\n            <option *ngFor=\"let refType of specification.types\" [ngValue]=\"refType.name\">{{ refType.name }}</option>\n          </optgroup>\n          <optgroup label=\"{{ include.alias }}\" *ngFor=\"let include of specification.imports\">\n            <ng-container *ngFor=\"let parentType of include.types\">\n              <option [ngValue]=\"include.alias + ':' + parentType.name\" *ngIf=\"parentType.name != type.name\">{{ parentType.name }}</option>\n            </ng-container>\n          </optgroup>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"property.type != 'generic' && property.reference == 'generic'\">\n      <label for=\"propertyGeneric\" class=\"col-sm-2 col-form-label fw-bold\">Generic</label>\n      <div class=\"col-sm-10\">\n        <input id=\"propertyGeneric\" name=\"propertyGeneric\" class=\"form-control\" [(ngModel)]=\"property.generic\">\n      </div>\n    </div>\n    <ng-container *ngIf=\"objectRef && property.type == 'object'\">\n      <ng-container [ngTemplateOutlet]=\"objectRef\" [ngTemplateOutletContext]=\"{$implicit: property}\"></ng-container>\n    </ng-container>\n    <ng-container *ngIf=\"mapRef && property.type == 'map'\">\n      <ng-container [ngTemplateOutlet]=\"mapRef\" [ngTemplateOutletContext]=\"{$implicit: property}\"></ng-container>\n    </ng-container>\n    <ng-container *ngIf=\"arrayRef && property.type == 'array'\">\n      <ng-container [ngTemplateOutlet]=\"arrayRef\" [ngTemplateOutletContext]=\"{$implicit: property}\"></ng-container>\n    </ng-container>\n    <ng-container *ngIf=\"stringRef && property.type == 'string'\">\n      <ng-container [ngTemplateOutlet]=\"stringRef\" [ngTemplateOutletContext]=\"{$implicit: property}\"></ng-container>\n    </ng-container>\n    <ng-container *ngIf=\"integerRef && property.type == 'integer'\">\n      <ng-container [ngTemplateOutlet]=\"integerRef\" [ngTemplateOutletContext]=\"{$implicit: property}\"></ng-container>\n    </ng-container>\n    <ng-container *ngIf=\"numberRef && property.type == 'number'\">\n      <ng-container [ngTemplateOutlet]=\"numberRef\" [ngTemplateOutletContext]=\"{$implicit: property}\"></ng-container>\n    </ng-container>\n    <ng-container *ngIf=\"booleanRef && property.type == 'boolean'\">\n      <ng-container [ngTemplateOutlet]=\"booleanRef\" [ngTemplateOutletContext]=\"{$implicit: property}\"></ng-container>\n    </ng-container>\n  </div>\n  <div class=\"modal-footer\">\n    <button type=\"button\" class=\"btn btn-outline-dark\" (click)=\"modal.close()\">Save</button>\n  </div>\n</ng-template>\n\n<ng-template #settingsModal let-modal>\n  <div class=\"modal-header\">\n    <h4 class=\"modal-title\">Settings</h4>\n    <button type=\"button\" class=\"btn-close\" aria-label=\"Close\" (click)=\"modal.dismiss()\"></button>\n  </div>\n  <div class=\"modal-body\" *ngIf=\"security\">\n    <div class=\"mb-3 row\">\n      <label for=\"settingsBaseUrl\" class=\"col-sm-2 col-form-label fw-bold\">Base-URL</label>\n      <div class=\"col-sm-10\">\n        <input [(ngModel)]=\"baseUrl\" id=\"settingsBaseUrl\" name=\"settingsBaseUrl\" class=\"form-control\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"settingsSecurityType\" class=\"col-sm-2 col-form-label fw-bold\">Type</label>\n      <div class=\"col-sm-10\">\n        <select [(ngModel)]=\"security.type\" id=\"settingsSecurityType\" name=\"settingsSecurityType\" class=\"form-select\">\n          <option ngValue=\"none\">None</option>\n          <option ngValue=\"httpBasic\">HTTP-Basic</option>\n          <option ngValue=\"httpBearer\">HTTP-Bearer</option>\n          <option ngValue=\"apiKey\">API-Key</option>\n          <option ngValue=\"oauth2\">OAuth2</option>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"security.type == 'apiKey'\">\n      <label for=\"settingsSecurityName\" class=\"col-sm-2 col-form-label fw-bold\">Name</label>\n      <div class=\"col-sm-10\">\n        <input [(ngModel)]=\"security.name\" id=\"settingsSecurityName\" name=\"settingsSecurityName\" class=\"form-control\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"security.type == 'apiKey'\">\n      <label for=\"settingsSecurityIn\" class=\"col-sm-2 col-form-label fw-bold\">In</label>\n      <div class=\"col-sm-10\">\n        <select [(ngModel)]=\"security.in\" id=\"settingsSecurityIn\" name=\"settingsSecurityIn\" class=\"form-select\">\n          <option ngValue=\"header\">Header</option>\n          <option ngValue=\"query\">Query</option>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"security.type == 'oauth2'\">\n      <label for=\"settingsSecurityTokenUrl\" class=\"col-sm-2 col-form-label fw-bold\">Token-URL</label>\n      <div class=\"col-sm-10\">\n        <input [(ngModel)]=\"security.tokenUrl\" id=\"settingsSecurityTokenUrl\" name=\"settingsSecurityTokenUrl\" class=\"form-control\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"security.type == 'oauth2'\">\n      <label for=\"settingsSecurityAuthorizationUrl\" class=\"col-sm-2 col-form-label fw-bold\">Authorization-URL</label>\n      <div class=\"col-sm-10\">\n        <input [(ngModel)]=\"security.authorizationUrl\" id=\"settingsSecurityAuthorizationUrl\" name=\"settingsSecurityAuthorizationUrl\" class=\"form-control\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"security.type == 'oauth2'\">\n      <label for=\"settingsSecurityScopes\" class=\"col-sm-2 col-form-label fw-bold\">Scopes</label>\n      <div class=\"col-sm-10\">\n        <input [(ngModel)]=\"security.scopes\" id=\"settingsSecurityScopes\" name=\"settingsSecurityScopes\" class=\"form-control\">\n      </div>\n    </div>\n  </div>\n  <div class=\"modal-footer\">\n    <button type=\"button\" class=\"btn btn-outline-dark\" (click)=\"modal.close()\">Save</button>\n  </div>\n</ng-template>\n\n<ng-template #includeModal let-modal>\n  <div class=\"modal-header\">\n    <h4 class=\"modal-title\">Include</h4>\n    <button type=\"button\" class=\"btn-close\" aria-label=\"Close\" (click)=\"modal.dismiss()\"></button>\n  </div>\n  <div class=\"modal-body\">\n    <div class=\"mb-3 row\">\n      <label for=\"includeAlias\" class=\"col-sm-2 col-form-label fw-bold\">Alias</label>\n      <div class=\"col-sm-10\">\n        <input id=\"includeAlias\" name=\"includeAlias\" class=\"form-control\" [(ngModel)]=\"include.alias\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"includeUrl\" class=\"col-sm-2 col-form-label fw-bold\">Url</label>\n      <div class=\"col-sm-10\">\n        <input id=\"includeUrl\" name=\"includeUrl\" type=\"url\" class=\"form-control\" [(ngModel)]=\"include.url\" aria-describedby=\"includeUrlHelpBlock\" />\n        <div id=\"includeUrlHelpBlock\" class=\"form-text\">\n          Url to a specification, this can be either a url <code>https://typeschema.org/specification.json</code>\n          or a document from the <a href=\"https://typehub.cloud/\">TypeHub</a> platform <code>typehub://apioo:developer&#64;1.0.1</code>.\n        </div>\n      </div>\n    </div>\n  </div>\n  <div class=\"modal-footer\">\n    <button type=\"button\" class=\"btn btn-outline-dark\" (click)=\"modal.close()\">Save</button>\n  </div>\n</ng-template>\n\n<ng-template #importModal let-modal>\n  <div class=\"modal-header\">\n    <h4 class=\"modal-title\">Import</h4>\n    <button type=\"button\" class=\"btn-close\" aria-label=\"Close\" (click)=\"modal.dismiss()\"></button>\n  </div>\n  <div class=\"modal-body\">\n    <div class=\"mb-3 row\">\n      <div class=\"col-sm-12\">\n        <select id=\"importType\" name=\"importType\" class=\"form-select\" [(ngModel)]=\"importType\">\n          <option ngValue=\"internal\">Internal (JSON)</option>\n          <option ngValue=\"typeapi\">TypeAPI (JSON)</option>\n          <option ngValue=\"typeschema\">TypeSchema (JSON)</option>\n          <option ngValue=\"openapi-json\">OpenAPI (JSON)</option>\n          <option ngValue=\"openapi-yaml\">OpenAPI (YAML)</option>\n          <option ngValue=\"jsonschema-json\">JsonSchema (JSON)</option>\n          <option ngValue=\"jsonschema-yaml\">JsonSchema (YAML)</option>\n          <option ngValue=\"raw-json\">RawJson (JSON)</option>\n          <option ngValue=\"raw-yaml\">RawYaml (YAML)</option>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <div class=\"col-sm-12\">\n        <textarea id=\"import\" name=\"import\" rows=\"20\" class=\"form-control\" [(ngModel)]=\"import\"></textarea>\n      </div>\n    </div>\n  </div>\n  <div class=\"modal-footer\">\n    <button type=\"button\" class=\"btn btn-primary\" (click)=\"modal.close()\">Import</button>\n  </div>\n</ng-template>\n\n<ng-template #exportModal let-modal>\n  <div class=\"modal-header\">\n    <h4 class=\"modal-title\">Export</h4>\n    <button type=\"button\" class=\"btn-close\" aria-label=\"Close\" (click)=\"modal.dismiss()\"></button>\n  </div>\n  <div class=\"modal-body\">\n    <div class=\"mb-3 row\">\n      <div class=\"col-sm-12\">\n        <textarea id=\"export\" name=\"export\" rows=\"20\" class=\"form-control\" [(ngModel)]=\"export\"></textarea>\n      </div>\n    </div>\n  </div>\n  <div class=\"modal-footer\">\n    <button type=\"button\" class=\"btn btn-primary\" (click)=\"modal.close()\">Close</button>\n  </div>\n</ng-template>\n\n<ng-template #tocOffCanvas let-offcanvas>\n  <div class=\"offcanvas-header\">\n    <h4 class=\"offcanvas-title\" id=\"offcanvas-basic-title\">TOC</h4>\n    <button type=\"button\" class=\"btn-close\" aria-label=\"Close\" (click)=\"offcanvas.dismiss('closed')\"></button>\n  </div>\n  <div class=\"offcanvas-body\">\n    <ul class=\"list-group\">\n      <ng-container *ngIf=\"operationEnabled\">\n        <li class=\"list-group-item disabled\">Operations</li>\n        <li *ngFor=\"let operation of specification.operations\" class=\"list-group-item\"><a (click)=\"offcanvas.close('operation-' + operation.name)\" role=\"button\" class=\"link-underline-primary\">{{operation.name}}</a></li>\n      </ng-container>\n      <li class=\"list-group-item disabled\">Types</li>\n      <li *ngFor=\"let type of specification.types\" class=\"list-group-item\"><a (click)=\"offcanvas.close('type-' + type.name)\" role=\"button\" class=\"link-underline-primary\">{{type.name}}</a></li>\n    </ul>\n    <div class=\"mt-4 text-secondary font-monospace\" *ngIf=\"!readonly\">\n      <p>Shortcuts:</p>\n      <ul class=\"list-unstyled\">\n        <ng-container *ngIf=\"operationEnabled\">\n          <li><kbd>ALT</kbd> + <kbd>q</kbd> Edit selected operation</li>\n          <li><kbd>ALT</kbd> + <kbd>a</kbd> Delete selected operation</li>\n          <li><kbd>ALT</kbd> + <kbd>y</kbd> New operation</li>\n          <li><kbd>ALT</kbd> + <kbd>w</kbd> Move up selected operation</li>\n          <li><kbd>ALT</kbd> + <kbd>s</kbd> Move down selected operation</li>\n        </ng-container>\n        <li><kbd>ALT</kbd> + <kbd>e</kbd> Edit selected type</li>\n        <li><kbd>ALT</kbd> + <kbd>d</kbd> Delete selected type</li>\n        <li><kbd>ALT</kbd> + <kbd>c</kbd> New type</li>\n        <li><kbd>ALT</kbd> + <kbd>r</kbd> Move up selected type</li>\n        <li><kbd>ALT</kbd> + <kbd>f</kbd> Move down selected type</li>\n      </ul>\n    </div>\n  </div>\n</ng-template>\n", styles: [""], dependencies: [{ kind: "component", type: i4.NgbAlert, selector: "ngb-alert", inputs: ["animation", "dismissible", "type"], outputs: ["closed"], exportAs: ["ngbAlert"] }, { kind: "directive", type: i5.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "directive", type: i5.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i5.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i5.NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }, { kind: "directive", type: i5.NgSwitch, selector: "[ngSwitch]", inputs: ["ngSwitch"] }, { kind: "directive", type: i5.NgSwitchCase, selector: "[ngSwitchCase]", inputs: ["ngSwitchCase"] }, { kind: "directive", type: i5.NgSwitchDefault, selector: "[ngSwitchDefault]" }, { kind: "directive", type: i2.NgSelectOption, selector: "option", inputs: ["ngValue", "value"] }, { kind: "directive", type: i2.ɵNgSelectMultipleOption, selector: "option", inputs: ["ngValue", "value"] }, { kind: "directive", type: i2.DefaultValueAccessor, selector: "input:not([type=checkbox])[formControlName],textarea[formControlName],input:not([type=checkbox])[formControl],textarea[formControl],input:not([type=checkbox])[ngModel],textarea[ngModel],[ngDefaultControl]" }, { kind: "directive", type: i2.CheckboxControlValueAccessor, selector: "input[type=checkbox][formControlName],input[type=checkbox][formControl],input[type=checkbox][ngModel]" }, { kind: "directive", type: i2.SelectControlValueAccessor, selector: "select:not([multiple])[formControlName],select:not([multiple])[formControl],select:not([multiple])[ngModel]", inputs: ["compareWith"] }, { kind: "directive", type: i2.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { kind: "directive", type: i2.NgModel, selector: "[ngModel]:not([formControlName]):not([formControl])", inputs: ["name", "disabled", "ngModel", "ngModelOptions"], outputs: ["ngModelChange"], exportAs: ["ngModel"] }, { kind: "component", type: ArgumentComponent, selector: "typeschema-arguments", inputs: ["data", "specification"], outputs: ["dataChange"] }, { kind: "component", type: LinkComponent, selector: "typeschema-link", inputs: ["type", "linkable"] }, { kind: "component", type: ImportComponent, selector: "typeschema-import", inputs: ["url"] }, { kind: "component", type: ThrowComponent, selector: "typeschema-throws", inputs: ["data", "specification", "contentTypes"], outputs: ["dataChange"] }, { kind: "pipe", type: i5.KeyValuePipe, name: "keyvalue" }] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: EditorComponent, decorators: [{
            type: Component,
            args: [{ selector: 'typeschema-editor', template: "\n<ng-container *ngIf=\"!readonly\">\n  <ngb-alert type=\"danger\" *ngIf=\"response && response.success === false\">{{ response.message }}</ngb-alert>\n  <ngb-alert type=\"success\" *ngIf=\"response && response.success === true\">{{ response.message }}</ngb-alert>\n\n  <div class=\"btn-group mb-3 sticky-top\" role=\"group\">\n    <button type=\"button\" class=\"btn btn-primary\" (click)=\"doSave()\" *ngIf=\"save.observed\">Save</button>\n    <button type=\"button\" class=\"btn btn-success\" (click)=\"openOperation(operationModal)\" *ngIf=\"operationEnabled\">New Operation</button>\n    <button type=\"button\" class=\"btn btn-success\" (click)=\"openType(typeModal)\">New Type</button>\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"openSettings(settingsModal)\" *ngIf=\"importEnabled\">Settings</button>\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"openInclude(includeModal)\" *ngIf=\"importEnabled\">Include</button>\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"openImport(importModal)\">Import</button>\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"openExport(exportModal)\">Export</button>\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"openToc(tocOffCanvas)\">TOC</button>\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"loadFromLocalStorage()\">Restore</button>\n  </div>\n</ng-container>\n<ng-container *ngIf=\"readonly\">\n  <div class=\"btn-group mb-3\" role=\"group\">\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"openExport(exportModal)\">Export</button>\n    <button type=\"button\" class=\"btn btn-light\" (click)=\"openToc(tocOffCanvas)\">TOC</button>\n  </div>\n</ng-container>\n\n<div class=\"text-center mt-5\" *ngIf=\"loading\">\n  <div class=\"spinner-border\" role=\"status\">\n    <span class=\"visually-hidden\">Loading...</span>\n  </div>\n</div>\n\n<div class=\"card editor-import mb-3\" *ngIf=\"specification.imports.length > 0\">\n  <div class=\"card-header\">\n    <div class=\"editor-import-headline font-monospace\">\n      <span class=\"badge text-bg-dark\"><span class=\"codicon codicon-cloud-download\"></span></span> Import\n    </div>\n  </div>\n  <div class=\"card-body\">\n    <table class=\"table table table-striped\">\n      <colgroup>\n        <col class=\"w-25\">\n        <col class=\"w-50\">\n        <col class=\"w-25\">\n      </colgroup>\n      <thead>\n      <tr>\n        <th>Alias</th>\n        <th>Url</th>\n        <th></th>\n      </tr>\n      </thead>\n      <tbody>\n      <tr *ngFor=\"let include of specification.imports; index as includeIndex\">\n        <td><span class=\"editor-import-alias\">{{ include.alias }}</span></td>\n        <td><span class=\"editor-import-url\"><typeschema-import [url]=\"include.url\"></typeschema-import></span></td>\n        <td>\n          <div class=\"btn-group float-end\" role=\"group\" *ngIf=\"!readonly\">\n            <a class=\"btn btn-danger\" (click)=\"deleteInclude(includeIndex)\"><span class=\"codicon codicon-close\"></span></a>\n          </div>\n        </td>\n      </tr>\n      </tbody>\n    </table>\n  </div>\n</div>\n\n<ng-container *ngIf=\"operationEnabled\">\n  <div *ngFor=\"let operation of specification.operations; index as operationIndex\" class=\"card editor-type mb-3\" [ngClass]=\"{'border-black': selectedOperation === operationIndex}\" id=\"operation-{{ operation.name }}\" (click)=\"selectOperation(operationIndex)\">\n    <div class=\"card-header\">\n      <div class=\"float-end text-end\">\n        <code>{{ operation.httpMethod }} {{ operation.httpPath }}</code>\n      </div>\n      <div class=\"editor-operation-name font-monospace\"><span class=\"badge text-bg-dark\"><span class=\"codicon codicon-symbol-method\"></span></span> {{ operation.name }}</div>\n      <div class=\"text-secondary editor-operation-description mt-2\" *ngIf=\"operation.description\">{{ operation.description }}</div>\n    </div>\n    <div class=\"card-body\">\n      <table class=\"table table-striped\" *ngIf=\"operation.arguments.length > 0 || operation.payload\">\n        <colgroup>\n          <col class=\"w-50\">\n          <col class=\"w-25\">\n          <col class=\"w-25\">\n        </colgroup>\n        <thead>\n        <tr>\n          <th>Name</th>\n          <th>Type</th>\n          <th></th>\n        </tr>\n        </thead>\n        <tbody>\n        <tr *ngFor=\"let argument of operation.arguments; index as argumentIndex\">\n          <td>\n            <span class=\"editor-operation-argument-name font-monospace\">{{ argument.name }}</span>\n          </td>\n          <td>\n            <span class=\"badge text-bg-primary\"><typeschema-link [type]=\"argument.type\" [linkable]=\"isLinkableType(argument.type)\"></typeschema-link></span>\n            <span class=\"badge text-bg-secondary ms-1\">{{ argument.in }}</span>\n          </td>\n          <td>\n            <div class=\"btn-group float-end\" role=\"group\" *ngIf=\"!readonly\">\n              <button type=\"button\" class=\"btn btn-secondary\" (click)=\"upArgument(operationIndex, argumentIndex)\"><i class=\"bi bi-arrow-up\"></i></button>\n              <button type=\"button\" class=\"btn btn-secondary\" (click)=\"downArgument(operationIndex, argumentIndex)\"><i class=\"bi bi-arrow-down\"></i></button>\n              <button type=\"button\" class=\"btn btn-danger\" (click)=\"deleteArgument(operationIndex, argumentIndex)\"><span class=\"codicon codicon-close\"></span></button>\n            </div>\n          </td>\n        </tr>\n        <tr *ngIf=\"operation.payload\">\n          <td>\n            <span class=\"editor-operation-argument-name font-monospace\">payload</span>\n          </td>\n          <td>\n            <span class=\"badge text-bg-primary\">\n              <ng-container *ngIf=\"operation.payloadShape === 'array'\">Array&lt;<typeschema-link [type]=\"operation.payload\" [linkable]=\"isLinkableType(operation.payload)\"></typeschema-link>&gt;</ng-container>\n              <ng-container *ngIf=\"operation.payloadShape === 'map'\">Map&lt;string, <typeschema-link [type]=\"operation.payload\" [linkable]=\"isLinkableType(operation.payload)\"></typeschema-link>&gt;</ng-container>\n              <ng-container *ngIf=\"operation.payloadShape === 'mime'\">{{ operation.payload }}</ng-container>\n              <ng-container *ngIf=\"operation.payloadShape !== 'array' && operation.payloadShape !== 'map' && operation.payloadShape !== 'mime'\"><typeschema-link [type]=\"operation.payload\" [linkable]=\"isLinkableType(operation.payload)\"></typeschema-link></ng-container>\n            </span>\n            <span class=\"badge text-bg-secondary ms-1\">body</span>\n          </td>\n          <td></td>\n        </tr>\n        </tbody>\n      </table>\n      <table class=\"table table-striped\" *ngIf=\"operation.httpCode || operation.throws.length > 0\">\n        <colgroup>\n          <col class=\"w-50\">\n          <col class=\"w-25\">\n          <col class=\"w-25\">\n        </colgroup>\n        <thead>\n        <tr>\n          <th>Code</th>\n          <th>Type</th>\n          <th></th>\n        </tr>\n        </thead>\n        <tbody>\n        <tr *ngIf=\"operation.httpCode === 204\">\n          <td><span class=\"badge text-bg-success\">{{ operation.httpCode }}</span></td>\n          <td><span class=\"badge text-bg-secondary\">-</span></td>\n          <td></td>\n        </tr>\n        <tr *ngIf=\"operation.httpCode && operation.return\">\n          <td><span class=\"badge text-bg-success\">{{ operation.httpCode }}</span></td>\n          <td>\n            <span class=\"badge text-bg-primary\">\n              <ng-container *ngIf=\"operation.returnShape === 'array'\">Array&lt;<typeschema-link [type]=\"operation.return\" [linkable]=\"isLinkableType(operation.return)\"></typeschema-link>&gt;</ng-container>\n              <ng-container *ngIf=\"operation.returnShape === 'map'\">Map&lt;string, <typeschema-link [type]=\"operation.return\" [linkable]=\"isLinkableType(operation.return)\"></typeschema-link>&gt;</ng-container>\n              <ng-container *ngIf=\"operation.returnShape === 'mime'\">{{ operation.return }}</ng-container>\n              <ng-container *ngIf=\"operation.returnShape !== 'array' && operation.returnShape !== 'map' && operation.returnShape !== 'mime'\"><typeschema-link [type]=\"operation.return\" [linkable]=\"isLinkableType(operation.return)\"></typeschema-link></ng-container>\n            </span>\n          </td>\n          <td></td>\n        </tr>\n        <tr *ngFor=\"let throw_ of operation.throws; index as throwIndex\">\n          <td><span class=\"badge text-bg-danger\">{{ throw_.code }}</span></td>\n          <td>\n            <span class=\"badge text-bg-primary\">\n              <ng-container *ngIf=\"throw_.typeShape === 'array'\">Array&lt;<typeschema-link [type]=\"throw_.type\" [linkable]=\"isLinkableType(throw_.type)\"></typeschema-link>&gt;</ng-container>\n              <ng-container *ngIf=\"throw_.typeShape === 'map'\">Map&lt;string, <typeschema-link [type]=\"throw_.type\" [linkable]=\"isLinkableType(throw_.type)\"></typeschema-link>&gt;</ng-container>\n              <ng-container *ngIf=\"throw_.typeShape === 'mime'\">{{ throw_.type }}</ng-container>\n              <ng-container *ngIf=\"throw_.typeShape !== 'array' && throw_.typeShape !== 'map' && throw_.typeShape !== 'mime'\"><typeschema-link [type]=\"throw_.type\" [linkable]=\"isLinkableType(throw_.type)\"></typeschema-link></ng-container>\n            </span>\n          </td>\n          <td>\n            <div class=\"btn-group float-end\" role=\"group\" *ngIf=\"!readonly\">\n              <button type=\"button\" class=\"btn btn-secondary\" (click)=\"upThrow(operationIndex, throwIndex)\"><i class=\"bi bi-arrow-up\"></i></button>\n              <button type=\"button\" class=\"btn btn-secondary\" (click)=\"downThrow(operationIndex, throwIndex)\"><i class=\"bi bi-arrow-down\"></i></button>\n              <button type=\"button\" class=\"btn btn-danger\" (click)=\"deleteThrow(operationIndex, throwIndex)\"><span class=\"codicon codicon-close\"></span></button>\n            </div>\n          </td>\n        </tr>\n        </tbody>\n      </table>\n    </div>\n    <div class=\"card-footer\" *ngIf=\"!readonly\">\n      <div class=\"btn-group\" role=\"group\">\n        <button type=\"button\" class=\"btn btn-primary\" (click)=\"editOperation(operationModal, operationIndex)\"><span class=\"codicon codicon-edit\"></span></button>\n        <button type=\"button\" class=\"btn btn-danger\" (click)=\"deleteOperation(operationIndex)\"><span class=\"codicon codicon-close\"></span></button>\n        <button type=\"button\" class=\"btn btn-secondary\" (click)=\"copyOperation(operationIndex)\"><span class=\"codicon codicon-copy\"></span></button>\n      </div>\n    </div>\n  </div>\n</ng-container>\n\n<div *ngFor=\"let type of specification.types; index as typeIndex\" class=\"card editor-type mb-3\" [ngClass]=\"{'border-primary': specification.root === typeIndex, 'border-black': selectedType === typeIndex}\" id=\"type-{{ type.name }}\" (click)=\"selectType(typeIndex)\">\n  <div class=\"card-header\">\n    <div class=\"editor-type-name font-monospace\"><span class=\"badge text-bg-dark\">\n      <span class=\"codicon codicon-symbol-structure\"></span></span> {{ type.name }}\n      <span *ngIf=\"type.parent\" class=\"float-end\"><span class=\"badge text-bg-primary\"><typeschema-link [type]=\"type.parent\" [linkable]=\"isLinkableType(type.parent)\"></typeschema-link></span></span>\n    </div>\n    <div class=\"text-secondary editor-type-description mt-2\" *ngIf=\"type.description\">{{ type.description }}</div>\n  </div>\n  <div class=\"card-body\">\n    <div [ngSwitch]=\"type.type\">\n      <ng-container *ngSwitchCase=\"'map'\">\n        <table class=\"table table table-striped editor-type-properties\">\n          <colgroup>\n            <col class=\"w-50\">\n            <col class=\"w-25\">\n            <col class=\"w-25\">\n          </colgroup>\n          <thead>\n          <tr>\n            <th>Name</th>\n            <th>Type</th>\n            <th></th>\n          </tr>\n          </thead>\n          <tbody>\n          <tr>\n            <td><div class=\"editor-type-property-name font-monospace\">*</div></td>\n            <td><span class=\"badge text-bg-primary\" *ngIf=\"type.reference\"><typeschema-link [type]=\"type.reference\" [linkable]=\"isLinkableType(type.reference)\"></typeschema-link></span></td>\n            <td></td>\n          </tr>\n          </tbody>\n        </table>\n      </ng-container>\n      <ng-container *ngSwitchDefault>\n        <div *ngIf=\"type.discriminator\">\n          <div>Discriminator: <span class=\"badge text-bg-primary\">{{ type.discriminator }}</span></div>\n          <table class=\"table table table-striped editor-type-properties\" *ngIf=\"type.mapping\">\n            <colgroup>\n              <col class=\"w-50\">\n              <col class=\"w-25\">\n              <col class=\"w-25\">\n            </colgroup>\n            <thead>\n            <tr>\n              <th>Value</th>\n              <th>Type</th>\n              <th></th>\n            </tr>\n            </thead>\n            <tbody>\n            <tr *ngFor=\"let mapping of type.mapping|keyvalue\">\n              <td><code>{{ mapping.value }}</code></td>\n              <td>\n                <div class=\"editor-type-property-name\">\n                  <span class=\"badge text-bg-primary\">\n                    <typeschema-link [type]=\"mapping.key\" [linkable]=\"isLinkableType(mapping.key)\"></typeschema-link>\n                  </span>\n                </div>\n              </td>\n              <td>\n                <div class=\"btn-group float-end\" role=\"group\" *ngIf=\"!readonly\">\n                  <button type=\"button\" class=\"btn btn-danger\" (click)=\"deleteTypeMapping(typeIndex, mapping.key)\"><span class=\"codicon codicon-close\"></span></button>\n                </div>\n              </td>\n            </tr>\n            </tbody>\n          </table>\n        </div>\n        <table class=\"table table table-striped editor-type-properties\">\n          <colgroup>\n            <col class=\"w-50\">\n            <col class=\"w-25\">\n            <col class=\"w-25\">\n          </colgroup>\n          <thead>\n          <tr>\n            <th>Name</th>\n            <th>Type</th>\n            <th></th>\n          </tr>\n          </thead>\n          <tbody>\n          <tr *ngFor=\"let property of type.properties; index as propertyIndex\">\n            <td>\n              <div class=\"editor-type-property-name font-monospace\">{{ property.name }}</div>\n              <div class=\"editor-type-property-description text-secondary mt-2\" *ngIf=\"property.description\">{{ property.description }}</div>\n            </td>\n            <td>\n              <span class=\"badge text-bg-primary\">{{ property.type }}</span>\n              <span class=\"badge text-bg-secondary ms-1\" *ngIf=\"property.reference\">\n                <typeschema-link [type]=\"property.reference\" [linkable]=\"isLinkableType(property.reference)\"></typeschema-link>\n              </span>\n              <span class=\"badge text-bg-secondary ms-1\" *ngIf=\"property.generic\">{{ property.generic }}</span>\n              <span class=\"badge text-bg-secondary ms-1\" *ngIf=\"property.format\">{{ property.format }}</span>\n            </td>\n            <td>\n              <div class=\"btn-group float-end\" role=\"group\" *ngIf=\"!readonly\">\n                <button type=\"button\" class=\"btn btn-secondary\" (click)=\"upProperty(typeIndex, propertyIndex)\"><i class=\"bi bi-arrow-up\"></i></button>\n                <button type=\"button\" class=\"btn btn-secondary\" (click)=\"downProperty(typeIndex, propertyIndex)\"><i class=\"bi bi-arrow-down\"></i></button>\n                <button type=\"button\" class=\"btn btn-primary\" (click)=\"editProperty(propertyModal, typeIndex, propertyIndex)\"><span class=\"codicon codicon-edit\"></span></button>\n                <button type=\"button\" class=\"btn btn-danger\" (click)=\"deleteProperty(typeIndex, propertyIndex)\"><span class=\"codicon codicon-close\"></span></button>\n              </div>\n            </td>\n          </tr>\n          </tbody>\n        </table>\n        <div class=\"btn-group\" role=\"group\" *ngIf=\"!readonly\">\n          <a class=\"btn btn-success\" (click)=\"openProperty(propertyModal, typeIndex)\">New Property</a>\n        </div>\n      </ng-container>\n    </div>\n  </div>\n  <div class=\"card-footer\" *ngIf=\"!readonly\">\n    <div class=\"btn-group\" role=\"group\">\n      <button type=\"button\" class=\"btn btn-primary\" (click)=\"editType(typeModal, typeIndex)\"><span class=\"codicon codicon-edit\"></span></button>\n      <button type=\"button\" class=\"btn btn-danger\" (click)=\"deleteType(typeIndex)\"><span class=\"codicon codicon-close\"></span></button>\n      <button type=\"button\" class=\"btn btn-secondary\" (click)=\"copyType(typeIndex)\"><span class=\"codicon codicon-copy\"></span></button>\n      <button *ngIf=\"specification.root !== typeIndex\" type=\"button\" class=\"btn btn-secondary\" (click)=\"setRoot(typeIndex)\">Primary</button>\n    </div>\n  </div>\n</div>\n\n<ng-template #operationModal let-modal>\n  <div class=\"modal-header\">\n    <h4 class=\"modal-title\">Operation</h4>\n    <button type=\"button\" class=\"btn-close\" aria-label=\"Close\" (click)=\"modal.dismiss()\"></button>\n  </div>\n  <div class=\"modal-body\">\n    <div class=\"mb-3 row\">\n      <label for=\"operationName\" class=\"col-sm-2 col-form-label fw-bold\">Name</label>\n      <div class=\"col-sm-10\">\n        <input id=\"operationName\" name=\"operationName\" class=\"form-control\" placeholder=\"my.operation.execute\" [(ngModel)]=\"operation.name\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"operationDescription\" class=\"col-sm-2 col-form-label fw-bold\">Description</label>\n      <div class=\"col-sm-10\">\n        <input id=\"operationDescription\" name=\"operationDescription\" class=\"form-control\" [(ngModel)]=\"operation.description\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"operationHttpMethod\" class=\"col-sm-2 col-form-label fw-bold\">HTTP Method</label>\n      <div class=\"col-sm-10\">\n        <select id=\"operationHttpMethod\" name=\"operationHttpMethod\" class=\"form-select\" [(ngModel)]=\"operation.httpMethod\">\n          <option ngValue=\"GET\">GET</option>\n          <option ngValue=\"POST\">POST</option>\n          <option ngValue=\"PUT\">PUT</option>\n          <option ngValue=\"PATCH\">PATCH</option>\n          <option ngValue=\"DELETE\">DELETE</option>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"operationHttpPath\" class=\"col-sm-2 col-form-label fw-bold\">HTTP Path</label>\n      <div class=\"col-sm-10\">\n        <input id=\"operationHttpPath\" name=\"operationHttpPath\" class=\"form-control\" placeholder=\"/my/endpoint\" [(ngModel)]=\"operation.httpPath\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"operationHttpCode\" class=\"col-sm-2 col-form-label fw-bold\">HTTP Code</label>\n      <div class=\"col-sm-10\">\n        <select id=\"operationHttpCode\" name=\"operationHttpCode\" class=\"form-select\" [(ngModel)]=\"operation.httpCode\">\n          <option [ngValue]=\"200\">200 OK</option>\n          <option [ngValue]=\"201\">201 Created</option>\n          <option [ngValue]=\"202\">202 Accepted</option>\n          <option [ngValue]=\"204\">204 No Content</option>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"operationArguments\" class=\"col-sm-2 col-form-label fw-bold\">Arguments</label>\n      <div class=\"col-sm-10\">\n        <typeschema-arguments [data]=\"operation.arguments\" [specification]=\"specification\" (dataChange)=\"operation.arguments = $event\"></typeschema-arguments>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"operation.httpMethod === 'POST' || operation.httpMethod === 'PUT' || operation.httpMethod === 'PATCH'\">\n      <label for=\"operationPayload\" class=\"col-sm-2 col-form-label fw-bold\">Payload</label>\n      <div class=\"col-sm-10\">\n        <div class=\"row mb-3\">\n          <div class=\"col-3\">\n            <select id=\"operationPayloadShape\" name=\"operationPayloadShape\" class=\"form-select\" [(ngModel)]=\"operation.payloadShape\">\n              <option [ngValue]=\"undefined\">Object</option>\n              <option ngValue=\"map\">Map</option>\n              <option ngValue=\"array\">Array</option>\n              <option ngValue=\"mime\">Content-Type</option>\n            </select>\n          </div>\n          <div class=\"col-9\">\n            <select *ngIf=\"operation.payloadShape !== 'mime'\" id=\"operationPayload\" name=\"operationPayload\" class=\"form-select\" [(ngModel)]=\"operation.payload\">\n              <optgroup label=\"self\">\n                <option *ngFor=\"let refType of specification.types\" [ngValue]=\"refType.name\">{{ refType.name }}</option>\n              </optgroup>\n              <optgroup label=\"{{ include.alias }}\" *ngFor=\"let include of specification.imports\">\n                <ng-container *ngFor=\"let parentType of include.types\">\n                  <option [ngValue]=\"include.alias + ':' + parentType.name\" *ngIf=\"parentType.name != type.name\">{{ parentType.name }}</option>\n                </ng-container>\n              </optgroup>\n            </select>\n            <select *ngIf=\"operation.payloadShape === 'mime'\" id=\"operationPayload\" name=\"operationPayload\" class=\"form-select\" [(ngModel)]=\"operation.payload\">\n              <option *ngFor=\"let contentType of contentTypes\" [ngValue]=\"contentType.value\">{{ contentType.name }}</option>\n            </select>\n          </div>\n        </div>\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"operationThrows\" class=\"col-sm-2 col-form-label fw-bold\">Throws</label>\n      <div class=\"col-sm-10\">\n        <typeschema-throws [data]=\"operation.throws\" [specification]=\"specification\" [contentTypes]=\"contentTypes\" (dataChange)=\"operation.throws = $event\"></typeschema-throws>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"operation.httpCode !== 204\">\n      <label for=\"operationReturn\" class=\"col-sm-2 col-form-label fw-bold\">Return</label>\n      <div class=\"col-sm-10\">\n        <div class=\"row mb-3\">\n          <div class=\"col-3\">\n            <select id=\"operationReturnShape\" name=\"operationReturnShape\" class=\"form-select\" [(ngModel)]=\"operation.returnShape\">\n              <option [ngValue]=\"undefined\">Object</option>\n              <option ngValue=\"map\">Map</option>\n              <option ngValue=\"array\">Array</option>\n              <option ngValue=\"mime\">Content-Type</option>\n            </select>\n          </div>\n          <div class=\"col-9\">\n            <select *ngIf=\"operation.returnShape !== 'mime'\" id=\"operationReturn\" name=\"operationReturn\" class=\"form-select\" [(ngModel)]=\"operation.return\">\n              <optgroup label=\"self\">\n                <option *ngFor=\"let refType of specification.types\" [ngValue]=\"refType.name\">{{ refType.name }}</option>\n              </optgroup>\n              <optgroup label=\"{{ include.alias }}\" *ngFor=\"let include of specification.imports\">\n                <ng-container *ngFor=\"let parentType of include.types\">\n                  <option [ngValue]=\"include.alias + ':' + parentType.name\" *ngIf=\"parentType.name != type.name\">{{ parentType.name }}</option>\n                </ng-container>\n              </optgroup>\n            </select>\n            <select *ngIf=\"operation.returnShape === 'mime'\" id=\"operationReturn\" name=\"operationReturn\" class=\"form-select\" [(ngModel)]=\"operation.return\">\n              <option *ngFor=\"let contentType of contentTypes\" [ngValue]=\"contentType.value\">{{ contentType.name }}</option>\n            </select>\n          </div>\n        </div>\n      </div>\n    </div>\n  </div>\n  <div class=\"modal-footer\">\n    <button type=\"button\" class=\"btn btn-outline-dark\" (click)=\"modal.close()\">Save</button>\n  </div>\n</ng-template>\n\n<ng-template #typeModal let-modal>\n  <div class=\"modal-header\">\n    <h4 class=\"modal-title\">Type</h4>\n    <button type=\"button\" class=\"btn-close\" aria-label=\"Close\" (click)=\"modal.dismiss()\"></button>\n  </div>\n  <div class=\"modal-body\">\n    <div class=\"mb-3 row\">\n      <label for=\"typeType\" class=\"col-sm-2 col-form-label fw-bold\">Type</label>\n      <div class=\"col-sm-10\">\n        <select id=\"typeType\" name=\"typeType\" class=\"form-select\" [(ngModel)]=\"type.type\">\n          <option ngValue=\"struct\">Struct</option>\n          <option ngValue=\"map\">Map</option>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"typeName\" class=\"col-sm-2 col-form-label fw-bold\">Name</label>\n      <div class=\"col-sm-10\">\n        <input id=\"typeName\" name=\"typeName\" class=\"form-control\" [(ngModel)]=\"type.name\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"typeDescription\" class=\"col-sm-2 col-form-label fw-bold\">Description</label>\n      <div class=\"col-sm-10\">\n        <input id=\"typeDescription\" name=\"typeDescription\" class=\"form-control\" [(ngModel)]=\"type.description\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"type.type === 'struct'\">\n      <label for=\"typeBase\" class=\"col-sm-2 col-form-label fw-bold\">Abstract</label>\n      <div class=\"col-sm-10\">\n        <div class=\"form-check form-switch mt-2\">\n          <input class=\"form-check-input\" type=\"checkbox\" role=\"switch\" id=\"typeBase\" [(ngModel)]=\"type.base\">\n        </div>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"type.type === 'struct'\">\n      <label for=\"typeParent\" class=\"col-sm-2 col-form-label fw-bold\">Parent</label>\n      <div class=\"col-sm-10\">\n        <select id=\"typeParent\" name=\"typeParent\" class=\"form-select\" [(ngModel)]=\"type.parent\" (ngModelChange)=\"updateGenerics()\">\n          <option [ngValue]=\"undefined\">No parent</option>\n          <optgroup label=\"Self\">\n            <ng-container *ngFor=\"let parentType of specification.types\">\n              <option [ngValue]=\"parentType.name\" *ngIf=\"parentType.name != type.name\">{{ parentType.name }}</option>\n            </ng-container>\n          </optgroup>\n          <optgroup label=\"{{include.alias}}\" *ngFor=\"let include of specification.imports\">\n            <ng-container *ngFor=\"let parentType of include.types\">\n              <option [ngValue]=\"include.alias + ':' + parentType.name\">{{ parentType.name }}</option>\n            </ng-container>\n          </optgroup>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"type.type === 'struct' && type.base\">\n      <label for=\"typeDiscriminator\" class=\"col-sm-2 col-form-label fw-bold\">Discriminator</label>\n      <div class=\"col-sm-10\">\n        <select id=\"typeDiscriminator\" name=\"typeDiscriminator\" class=\"form-select\" [(ngModel)]=\"type.discriminator\" (ngModelChange)=\"updateDiscriminator()\">\n          <option [ngValue]=\"undefined\">No discriminator</option>\n          <ng-container *ngFor=\"let property of type.properties\">\n            <option [ngValue]=\"property.name\">{{ property.name }}</option>\n          </ng-container>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"type.type === 'struct' && type.discriminator && type.mapping && (type.mapping | keyvalue)?.length\">\n      <label class=\"col-sm-2 col-form-label fw-bold\">Mapping</label>\n      <div class=\"col-sm-10\">\n        <div *ngFor=\"let child of children\">\n          <div class=\"input-group mb-3\">\n            <input id=\"typeMapping-{{ child }}\" name=\"typeMapping-{{ child }}\" type=\"text\" class=\"form-control\" [(ngModel)]=\"type.mapping[child]\" placeholder=\"Value of the discriminator property\">\n            <span class=\"input-group-text\">{{ child }}</span>\n          </div>\n        </div>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"type.type === 'struct' && type.template && (type.template | keyvalue)?.length\">\n      <label class=\"col-sm-2 col-form-label fw-bold\">Template</label>\n      <div class=\"col-sm-10\">\n        <div *ngFor=\"let generic of generics\">\n          <div class=\"input-group mb-3\">\n            <select id=\"typeTemplate-{{ generic }}\" name=\"typeTemplate-{{ generic }}\" class=\"form-select\" [(ngModel)]=\"type.template[generic]\">\n              <option>No type</option>\n              <ng-container *ngFor=\"let type of specification.types\">\n                <option [ngValue]=\"type.name\">{{ type.name }}</option>\n              </ng-container>\n            </select>\n            <span class=\"input-group-text\">{{ generic }}</span>\n          </div>\n        </div>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"type.type === 'map'\">\n      <label for=\"typeReference\" class=\"col-sm-2 col-form-label fw-bold\">Reference</label>\n      <div class=\"col-sm-10\">\n        <select id=\"typeReference\" name=\"typeReference\" class=\"form-select\" [(ngModel)]=\"type.reference\">\n          <optgroup label=\"scalar\">\n            <option ngValue=\"string\">String</option>\n            <option ngValue=\"integer\">Integer</option>\n            <option ngValue=\"number\">Number</option>\n            <option ngValue=\"boolean\">Boolean</option>\n            <option ngValue=\"any\">Any</option>\n            <option ngValue=\"generic\">Generic</option>\n          </optgroup>\n          <optgroup label=\"self\">\n            <option *ngFor=\"let refType of specification.types\" [ngValue]=\"refType.name\">{{ refType.name }}</option>\n          </optgroup>\n          <optgroup label=\"{{ include.alias }}\" *ngFor=\"let include of specification.imports\">\n            <ng-container *ngFor=\"let parentType of include.types\">\n              <option [ngValue]=\"include.alias + ':' + parentType.name\" *ngIf=\"parentType.name != type.name\">{{ parentType.name }}</option>\n            </ng-container>\n          </optgroup>\n        </select>\n      </div>\n    </div>\n  </div>\n  <div class=\"modal-footer\">\n    <button type=\"button\" class=\"btn btn-outline-dark\" (click)=\"modal.close()\">Save</button>\n  </div>\n</ng-template>\n\n<ng-template #propertyModal let-modal>\n  <div class=\"modal-header\">\n    <h4 class=\"modal-title\">Property</h4>\n    <button type=\"button\" class=\"btn-close\" aria-label=\"Close\" (click)=\"modal.dismiss()\"></button>\n  </div>\n  <div class=\"modal-body\">\n    <div class=\"mb-3 row\">\n      <label for=\"propertyName\" class=\"col-sm-2 col-form-label fw-bold\">Name</label>\n      <div class=\"col-sm-10\">\n        <input id=\"propertyName\" name=\"propertyName\" class=\"form-control\" [(ngModel)]=\"property.name\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"propertyDescription\" class=\"col-sm-2 col-form-label fw-bold\">Description</label>\n      <div class=\"col-sm-10\">\n        <input id=\"propertyDescription\" name=\"propertyDescription\" class=\"form-control\" [(ngModel)]=\"property.description\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"propertyType\" class=\"col-sm-2 col-form-label fw-bold\">Type</label>\n      <div class=\"col-sm-10\">\n        <select id=\"propertyType\" name=\"propertyType\" class=\"form-select\" [(ngModel)]=\"property.type\">\n          <optgroup label=\"Scalar\">\n            <option ngValue=\"string\">String</option>\n            <option ngValue=\"integer\">Integer</option>\n            <option ngValue=\"number\">Number</option>\n            <option ngValue=\"boolean\">Boolean</option>\n          </optgroup>\n          <optgroup label=\"Complex\">\n            <option ngValue=\"object\">Object</option>\n            <option ngValue=\"map\">Map</option>\n            <option ngValue=\"array\">Array</option>\n            <option ngValue=\"any\">Any</option>\n            <option ngValue=\"generic\">Generic</option>\n          </optgroup>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"property.type == 'string'\">\n      <label for=\"propertyFormat\" class=\"col-sm-2 col-form-label fw-bold\">Format</label>\n      <div class=\"col-sm-10\">\n        <select id=\"propertyFormat\" name=\"propertyFormat\" class=\"form-select\" [(ngModel)]=\"property.format\">\n          <option [ngValue]=\"undefined\">No Format</option>\n          <option *ngIf=\"property.type == 'string'\" ngValue=\"date\">Date</option>\n          <option *ngIf=\"property.type == 'string'\" ngValue=\"date-time\">Date-Time</option>\n          <option *ngIf=\"property.type == 'string'\" ngValue=\"time\">Time</option>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"property.type == 'generic'\">\n      <label for=\"propertyGeneric\" class=\"col-sm-2 col-form-label fw-bold\">Generic</label>\n      <div class=\"col-sm-10\">\n        <input id=\"propertyGeneric\" name=\"propertyGeneric\" class=\"form-control\" [(ngModel)]=\"property.generic\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"property.type == 'object' || property.type == 'array' || property.type == 'map'\">\n      <label for=\"propertyReference\" class=\"col-sm-2 col-form-label fw-bold\">Reference</label>\n      <div class=\"col-sm-10\">\n        <select id=\"propertyReference\" name=\"propertyReference\" class=\"form-select\" [(ngModel)]=\"property.reference\" (ngModelChange)=\"updatePropertyReference()\">\n          <optgroup label=\"scalar\" *ngIf=\"property.type != 'object'\">\n            <option ngValue=\"string\">String</option>\n            <option ngValue=\"integer\">Integer</option>\n            <option ngValue=\"number\">Number</option>\n            <option ngValue=\"boolean\">Boolean</option>\n            <option ngValue=\"any\">Any</option>\n            <option ngValue=\"generic\">Generic</option>\n          </optgroup>\n          <optgroup label=\"self\">\n            <option *ngFor=\"let refType of specification.types\" [ngValue]=\"refType.name\">{{ refType.name }}</option>\n          </optgroup>\n          <optgroup label=\"{{ include.alias }}\" *ngFor=\"let include of specification.imports\">\n            <ng-container *ngFor=\"let parentType of include.types\">\n              <option [ngValue]=\"include.alias + ':' + parentType.name\" *ngIf=\"parentType.name != type.name\">{{ parentType.name }}</option>\n            </ng-container>\n          </optgroup>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"property.type != 'generic' && property.reference == 'generic'\">\n      <label for=\"propertyGeneric\" class=\"col-sm-2 col-form-label fw-bold\">Generic</label>\n      <div class=\"col-sm-10\">\n        <input id=\"propertyGeneric\" name=\"propertyGeneric\" class=\"form-control\" [(ngModel)]=\"property.generic\">\n      </div>\n    </div>\n    <ng-container *ngIf=\"objectRef && property.type == 'object'\">\n      <ng-container [ngTemplateOutlet]=\"objectRef\" [ngTemplateOutletContext]=\"{$implicit: property}\"></ng-container>\n    </ng-container>\n    <ng-container *ngIf=\"mapRef && property.type == 'map'\">\n      <ng-container [ngTemplateOutlet]=\"mapRef\" [ngTemplateOutletContext]=\"{$implicit: property}\"></ng-container>\n    </ng-container>\n    <ng-container *ngIf=\"arrayRef && property.type == 'array'\">\n      <ng-container [ngTemplateOutlet]=\"arrayRef\" [ngTemplateOutletContext]=\"{$implicit: property}\"></ng-container>\n    </ng-container>\n    <ng-container *ngIf=\"stringRef && property.type == 'string'\">\n      <ng-container [ngTemplateOutlet]=\"stringRef\" [ngTemplateOutletContext]=\"{$implicit: property}\"></ng-container>\n    </ng-container>\n    <ng-container *ngIf=\"integerRef && property.type == 'integer'\">\n      <ng-container [ngTemplateOutlet]=\"integerRef\" [ngTemplateOutletContext]=\"{$implicit: property}\"></ng-container>\n    </ng-container>\n    <ng-container *ngIf=\"numberRef && property.type == 'number'\">\n      <ng-container [ngTemplateOutlet]=\"numberRef\" [ngTemplateOutletContext]=\"{$implicit: property}\"></ng-container>\n    </ng-container>\n    <ng-container *ngIf=\"booleanRef && property.type == 'boolean'\">\n      <ng-container [ngTemplateOutlet]=\"booleanRef\" [ngTemplateOutletContext]=\"{$implicit: property}\"></ng-container>\n    </ng-container>\n  </div>\n  <div class=\"modal-footer\">\n    <button type=\"button\" class=\"btn btn-outline-dark\" (click)=\"modal.close()\">Save</button>\n  </div>\n</ng-template>\n\n<ng-template #settingsModal let-modal>\n  <div class=\"modal-header\">\n    <h4 class=\"modal-title\">Settings</h4>\n    <button type=\"button\" class=\"btn-close\" aria-label=\"Close\" (click)=\"modal.dismiss()\"></button>\n  </div>\n  <div class=\"modal-body\" *ngIf=\"security\">\n    <div class=\"mb-3 row\">\n      <label for=\"settingsBaseUrl\" class=\"col-sm-2 col-form-label fw-bold\">Base-URL</label>\n      <div class=\"col-sm-10\">\n        <input [(ngModel)]=\"baseUrl\" id=\"settingsBaseUrl\" name=\"settingsBaseUrl\" class=\"form-control\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"settingsSecurityType\" class=\"col-sm-2 col-form-label fw-bold\">Type</label>\n      <div class=\"col-sm-10\">\n        <select [(ngModel)]=\"security.type\" id=\"settingsSecurityType\" name=\"settingsSecurityType\" class=\"form-select\">\n          <option ngValue=\"none\">None</option>\n          <option ngValue=\"httpBasic\">HTTP-Basic</option>\n          <option ngValue=\"httpBearer\">HTTP-Bearer</option>\n          <option ngValue=\"apiKey\">API-Key</option>\n          <option ngValue=\"oauth2\">OAuth2</option>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"security.type == 'apiKey'\">\n      <label for=\"settingsSecurityName\" class=\"col-sm-2 col-form-label fw-bold\">Name</label>\n      <div class=\"col-sm-10\">\n        <input [(ngModel)]=\"security.name\" id=\"settingsSecurityName\" name=\"settingsSecurityName\" class=\"form-control\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"security.type == 'apiKey'\">\n      <label for=\"settingsSecurityIn\" class=\"col-sm-2 col-form-label fw-bold\">In</label>\n      <div class=\"col-sm-10\">\n        <select [(ngModel)]=\"security.in\" id=\"settingsSecurityIn\" name=\"settingsSecurityIn\" class=\"form-select\">\n          <option ngValue=\"header\">Header</option>\n          <option ngValue=\"query\">Query</option>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"security.type == 'oauth2'\">\n      <label for=\"settingsSecurityTokenUrl\" class=\"col-sm-2 col-form-label fw-bold\">Token-URL</label>\n      <div class=\"col-sm-10\">\n        <input [(ngModel)]=\"security.tokenUrl\" id=\"settingsSecurityTokenUrl\" name=\"settingsSecurityTokenUrl\" class=\"form-control\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"security.type == 'oauth2'\">\n      <label for=\"settingsSecurityAuthorizationUrl\" class=\"col-sm-2 col-form-label fw-bold\">Authorization-URL</label>\n      <div class=\"col-sm-10\">\n        <input [(ngModel)]=\"security.authorizationUrl\" id=\"settingsSecurityAuthorizationUrl\" name=\"settingsSecurityAuthorizationUrl\" class=\"form-control\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\" *ngIf=\"security.type == 'oauth2'\">\n      <label for=\"settingsSecurityScopes\" class=\"col-sm-2 col-form-label fw-bold\">Scopes</label>\n      <div class=\"col-sm-10\">\n        <input [(ngModel)]=\"security.scopes\" id=\"settingsSecurityScopes\" name=\"settingsSecurityScopes\" class=\"form-control\">\n      </div>\n    </div>\n  </div>\n  <div class=\"modal-footer\">\n    <button type=\"button\" class=\"btn btn-outline-dark\" (click)=\"modal.close()\">Save</button>\n  </div>\n</ng-template>\n\n<ng-template #includeModal let-modal>\n  <div class=\"modal-header\">\n    <h4 class=\"modal-title\">Include</h4>\n    <button type=\"button\" class=\"btn-close\" aria-label=\"Close\" (click)=\"modal.dismiss()\"></button>\n  </div>\n  <div class=\"modal-body\">\n    <div class=\"mb-3 row\">\n      <label for=\"includeAlias\" class=\"col-sm-2 col-form-label fw-bold\">Alias</label>\n      <div class=\"col-sm-10\">\n        <input id=\"includeAlias\" name=\"includeAlias\" class=\"form-control\" [(ngModel)]=\"include.alias\">\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <label for=\"includeUrl\" class=\"col-sm-2 col-form-label fw-bold\">Url</label>\n      <div class=\"col-sm-10\">\n        <input id=\"includeUrl\" name=\"includeUrl\" type=\"url\" class=\"form-control\" [(ngModel)]=\"include.url\" aria-describedby=\"includeUrlHelpBlock\" />\n        <div id=\"includeUrlHelpBlock\" class=\"form-text\">\n          Url to a specification, this can be either a url <code>https://typeschema.org/specification.json</code>\n          or a document from the <a href=\"https://typehub.cloud/\">TypeHub</a> platform <code>typehub://apioo:developer&#64;1.0.1</code>.\n        </div>\n      </div>\n    </div>\n  </div>\n  <div class=\"modal-footer\">\n    <button type=\"button\" class=\"btn btn-outline-dark\" (click)=\"modal.close()\">Save</button>\n  </div>\n</ng-template>\n\n<ng-template #importModal let-modal>\n  <div class=\"modal-header\">\n    <h4 class=\"modal-title\">Import</h4>\n    <button type=\"button\" class=\"btn-close\" aria-label=\"Close\" (click)=\"modal.dismiss()\"></button>\n  </div>\n  <div class=\"modal-body\">\n    <div class=\"mb-3 row\">\n      <div class=\"col-sm-12\">\n        <select id=\"importType\" name=\"importType\" class=\"form-select\" [(ngModel)]=\"importType\">\n          <option ngValue=\"internal\">Internal (JSON)</option>\n          <option ngValue=\"typeapi\">TypeAPI (JSON)</option>\n          <option ngValue=\"typeschema\">TypeSchema (JSON)</option>\n          <option ngValue=\"openapi-json\">OpenAPI (JSON)</option>\n          <option ngValue=\"openapi-yaml\">OpenAPI (YAML)</option>\n          <option ngValue=\"jsonschema-json\">JsonSchema (JSON)</option>\n          <option ngValue=\"jsonschema-yaml\">JsonSchema (YAML)</option>\n          <option ngValue=\"raw-json\">RawJson (JSON)</option>\n          <option ngValue=\"raw-yaml\">RawYaml (YAML)</option>\n        </select>\n      </div>\n    </div>\n    <div class=\"mb-3 row\">\n      <div class=\"col-sm-12\">\n        <textarea id=\"import\" name=\"import\" rows=\"20\" class=\"form-control\" [(ngModel)]=\"import\"></textarea>\n      </div>\n    </div>\n  </div>\n  <div class=\"modal-footer\">\n    <button type=\"button\" class=\"btn btn-primary\" (click)=\"modal.close()\">Import</button>\n  </div>\n</ng-template>\n\n<ng-template #exportModal let-modal>\n  <div class=\"modal-header\">\n    <h4 class=\"modal-title\">Export</h4>\n    <button type=\"button\" class=\"btn-close\" aria-label=\"Close\" (click)=\"modal.dismiss()\"></button>\n  </div>\n  <div class=\"modal-body\">\n    <div class=\"mb-3 row\">\n      <div class=\"col-sm-12\">\n        <textarea id=\"export\" name=\"export\" rows=\"20\" class=\"form-control\" [(ngModel)]=\"export\"></textarea>\n      </div>\n    </div>\n  </div>\n  <div class=\"modal-footer\">\n    <button type=\"button\" class=\"btn btn-primary\" (click)=\"modal.close()\">Close</button>\n  </div>\n</ng-template>\n\n<ng-template #tocOffCanvas let-offcanvas>\n  <div class=\"offcanvas-header\">\n    <h4 class=\"offcanvas-title\" id=\"offcanvas-basic-title\">TOC</h4>\n    <button type=\"button\" class=\"btn-close\" aria-label=\"Close\" (click)=\"offcanvas.dismiss('closed')\"></button>\n  </div>\n  <div class=\"offcanvas-body\">\n    <ul class=\"list-group\">\n      <ng-container *ngIf=\"operationEnabled\">\n        <li class=\"list-group-item disabled\">Operations</li>\n        <li *ngFor=\"let operation of specification.operations\" class=\"list-group-item\"><a (click)=\"offcanvas.close('operation-' + operation.name)\" role=\"button\" class=\"link-underline-primary\">{{operation.name}}</a></li>\n      </ng-container>\n      <li class=\"list-group-item disabled\">Types</li>\n      <li *ngFor=\"let type of specification.types\" class=\"list-group-item\"><a (click)=\"offcanvas.close('type-' + type.name)\" role=\"button\" class=\"link-underline-primary\">{{type.name}}</a></li>\n    </ul>\n    <div class=\"mt-4 text-secondary font-monospace\" *ngIf=\"!readonly\">\n      <p>Shortcuts:</p>\n      <ul class=\"list-unstyled\">\n        <ng-container *ngIf=\"operationEnabled\">\n          <li><kbd>ALT</kbd> + <kbd>q</kbd> Edit selected operation</li>\n          <li><kbd>ALT</kbd> + <kbd>a</kbd> Delete selected operation</li>\n          <li><kbd>ALT</kbd> + <kbd>y</kbd> New operation</li>\n          <li><kbd>ALT</kbd> + <kbd>w</kbd> Move up selected operation</li>\n          <li><kbd>ALT</kbd> + <kbd>s</kbd> Move down selected operation</li>\n        </ng-container>\n        <li><kbd>ALT</kbd> + <kbd>e</kbd> Edit selected type</li>\n        <li><kbd>ALT</kbd> + <kbd>d</kbd> Delete selected type</li>\n        <li><kbd>ALT</kbd> + <kbd>c</kbd> New type</li>\n        <li><kbd>ALT</kbd> + <kbd>r</kbd> Move up selected type</li>\n        <li><kbd>ALT</kbd> + <kbd>f</kbd> Move down selected type</li>\n      </ul>\n    </div>\n  </div>\n</ng-template>\n" }]
        }], ctorParameters: () => [{ type: ImportService }, { type: ResolverService }, { type: BCLayerService }, { type: i4.NgbModal }, { type: i4.NgbOffcanvas }, { type: i5.ViewportScroller }], propDecorators: { specification: [{
                type: Input
            }], operationEnabled: [{
                type: Input
            }], importEnabled: [{
                type: Input
            }], readonly: [{
                type: Input
            }], id: [{
                type: Input
            }], objectRef: [{
                type: Input
            }], arrayRef: [{
                type: Input
            }], mapRef: [{
                type: Input
            }], stringRef: [{
                type: Input
            }], integerRef: [{
                type: Input
            }], numberRef: [{
                type: Input
            }], booleanRef: [{
                type: Input
            }], save: [{
                type: Output
            }], change: [{
                type: Output
            }], operationModalRef: [{
                type: ViewChild,
                args: ['operationModal']
            }], typeModalRef: [{
                type: ViewChild,
                args: ['typeModal']
            }], handleKeyboardEvent: [{
                type: HostListener,
                args: ['document:keydown', ['$event']]
            }] } });

class EditorConfirmSaveGuard {
    canDeactivate(component, currentRoute, currentState, nextState) {
        if (component instanceof EditorComponent) {
            if (component.dirty) {
                if (confirm('The editor contains unsaved changes. Please confirm that you actually want to lose these changes.')) {
                    return true;
                }
                else {
                    return false;
                }
            }
        }
        return true;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: EditorConfirmSaveGuard, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
    static ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: EditorConfirmSaveGuard, providedIn: 'root' });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: EditorConfirmSaveGuard, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }] });

class TypeschemaEditorModule {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TypeschemaEditorModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
    static ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "18.2.13", ngImport: i0, type: TypeschemaEditorModule, declarations: [ArgumentComponent,
            LinkComponent,
            ImportComponent,
            ThrowComponent,
            EditorComponent], imports: [NgbModule,
            CommonModule,
            FormsModule], exports: [EditorComponent] });
    static ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TypeschemaEditorModule, imports: [NgbModule,
            CommonModule,
            FormsModule] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: TypeschemaEditorModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        ArgumentComponent,
                        LinkComponent,
                        ImportComponent,
                        ThrowComponent,
                        EditorComponent
                    ],
                    imports: [
                        NgbModule,
                        CommonModule,
                        FormsModule
                    ],
                    exports: [
                        EditorComponent
                    ]
                }]
        }] });

/*
 * Public API Surface of typeschema-angular-editor
 */

/**
 * Generated bundle index. Do not edit.
 */

export { EditorComponent, EditorConfirmSaveGuard, ExportService, ImportService, TypeschemaEditorModule };
//# sourceMappingURL=ngx-typeschema-editor.mjs.map
